version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
preliminaries

global post_data_loaded "../external/compiled/CompiledStayLevelData"
global pre_data_loaded "../external/pre_pps/pre_pps_data"

global LTCH_aha = "../external/AHA/aha_append_ltch"
global ACH_aha = "../external/AHA/aha_append_ach"
global SNF_IRF_aha = "../external/AHA/aha_append_snf_irf"

program main
    share_fp_np
    export_threshold_tables
    count_repeats
    avg_los_post_thres
    sample_size
    hco_payments
    pac_share_deaths
end

program share_fp_np
    local year 2008
    foreach dataset in ACH LTCH SNF_IRF {
        use aha_data_year cntrl_fp cntrl_npngov cntrl_gov ///
            if aha_data_year==`year' using ${`dataset'_aha}, clear
        foreach share in fp npngov gov {
            sum cntrl_`share'
            matrix hosp`year'level`dataset' = nullmat(hosp`year'level`dataset') \ r(mean)
        }
    }
    matrix hosp`year'level_fpnp = hosp`year'levelACH, hosp`year'levelLTCH, hosp`year'levelSNF_IRF
    matrix_to_txt, saving(../output/tables/hosp`year'level_fpnp.txt) ///
        mat(hosp`year'level_fpnp) format(%20.4f) ///
        title(<tab:hosp`year'level_fpnp>) replace
end

program export_threshold_tables
    use Kink rel_los using ${post_data_loaded}, clear
    sum Kink
    ooo  Average SSO threshold PPS
    di "Average threshold in post-period sample is: `r(mean)'"
    foreach t in 15 45 {
        gen past_`t' = rel_los > `t'
        sum past_`t'
        local past_`t'_post = `r(mean)'
    }

    gen freq=1
    collapse (sum) freq, by(Kink)
    egen total=total(freq)
    gen pct = freq/total
    gen cum = sum(pct)
    drop total
    dta_to_txt, saving(../output/tables/thresholds_post.txt) ///
        title(<tab:thresholds_post>) replace noquote nonames

    use Kink rel_los util_day using ${pre_data_loaded}, clear
    sum Kink
    ooo Average SSO threshold pre-PPS
    di "Average threshold in pre-period sample is: `r(mean)'"
    foreach t in 15 45 {
        gen past_`t' = rel_los > `t'
        sum past_`t'
        local past_`t'_pre = `r(mean)'
    }
    gen leftofkink = util_day < Kink
    gen freq=1
    collapse (sum) freq, by(leftofkink)
    egen total=total(freq)
    gen pct = freq/total
    drop total
    dta_to_txt, saving(../output/tables/leftofkink_pre.txt) ///
        title(<tab:leftofkink_pre>) replace noquote nonames

    use rel_los using ${pre_data_loaded}, clear
    append using ${post_data_loaded}, keep(rel_los)
    foreach t in 15 45 {
        gen past_`t' = rel_los > `t'
        sum past_`t'
        local past_`t'_pooled = `r(mean)'
    }
    matrix right_tail = (`past_15_pre', `past_15_post', `past_15_pooled') \ ///
        (`past_45_pre', `past_45_post', `past_45_pooled')
    matrix_to_txt, saving(../output/tables/right_tail.txt) matrix(right_tail) format(%20.4f) ///
        title(<tab:right_tail>) replace

end

program count_repeats
    foreach period in pre post {
        use bene_id using ${`period'_data_loaded}, clear
        duplicates tag bene_id, gen(dup)
        gen isdup= dup>0
        keep bene_id isdup
        duplicates drop
        gen freq=1
        collapse (sum) freq, by(isdup)
        egen total=total(freq)
        gen pct = freq/total
        drop total
        dta_to_txt, saving(../output/tables/duplicates_`period'.txt) ///
            title(<tab:duplicates_`period'>) replace noquote nonames
    }
end

program avg_los_post_thres
    use bene_id Kink util_day using ${post_data_loaded}, clear
    keep if util_day >= Kink
    gen days_post_thres = util_day - Kink
    sum days_post_thres
    local avg_stay `r(mean)'
    ooo Average Days Post Threshold
    di("Average num days post threshold stayed conditional on reaching threshold is `avg_stay'")
end

program sample_size
    foreach period in pre post {
        use "${`period'_data_loaded}", clear
        count
        ooo `period' PPS sample size
        di("N stays in `period' period: `r(N)'")
    }
end

program hco_payments
    use ../external/compiled/CompiledStayLevelData, clear
    gen hco = cgs_costoutlier > 0
    sum hco
    local hco_share : di %5.4f `r(mean)'
    ooo Share Stays w HCO
    di("Share of post-period stays with HCO payments: `hco_share'")
    qui sum cgs_costoutlier if hco, det
    local med_pay : di %6.2f `r(p50)'
    ooo Median HCO payment
    di("Median HCO payment if any: `med_pay'")
end

program pac_share_deaths
    use admsndt util_day dschrgdt bene_id medparid ltch snf irf prvdrnum ///
        using ../external_links/medpar/medpar_append_post, clear
    keep if ltch == 1 | snf == 1 | irf == 1
    keep if (year(dschrgdt) > 2007 & year(dschrgdt) < 2012) | ///
       (year(dschrgdt) == 2007 & month(dschrgdt) >= 10) | ///
       (year(dschrgdt) == 2012 & month(dschrgdt) <= 7)
    merge m:1 bene_id using ../external_links/deaths/deathlist1998_2014, ///
        keep(2 3) keepusing(death_dt)
    * Restrict to deaths within 1 month after start and end dates of our sample
    keep if (year(death_dt) > 2007 & year(death_dt) < 2012) | ///
        (year(death_dt) == 2007 & month(death_dt >= 11)) | ///
        (year(death_dt) == 2012 & month(death_dt) <= 8)
    gen distance_to_death = death_dt - dschrgdt
    replace distance_to_death = . if distance_to_death<0
    gen pacspell_30days = distance_to_death<=30
    assert pacspell_30days==0 if _merge==2
    collapse (max) pacspell_30days, by(bene_id)
    sum pacspell_30days
    local share_death_pac_30day : di %5.4f `r(mean)'
    di("Fraction of deaths involving a PAC spell within 30 days of death: `share_death_pac_30day'")
end

* EXECUTE
main
