version 14
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado
preliminaries

global min_day = -15
global max_day = 45

global test_bw = 3

set seed 1

program main
    estimate_rd
    export_rd_coeffs
    * can only run perturbation_test when we've generated the global variables in estimate_rd
    run_perturbation_test_regs
    export_perturb_coeffs
end

program estimate_rd
    foreach period in pre post  {
        foreach n in 1 30 {
            foreach samp in pooled forprof nonprof {
                use  death_`n'day day right_sso day_r_sso day_sq day_sq_r_sso drgcd ///
                    using ../external/mort_haz/`period'_panel_`samp', clear

                foreach form in 1 2 {
                    local xvars right_sso day day_r_sso
                    if "`form'"=="2" {
                        local xvars `xvars' day_sq day_sq_r_sso
                    }
                
                    foreach bw in 3 5 10 {
                        reg death_`n'day `xvars' if abs(day)<=`bw', cluster(drgcd) 
                        mat table = r(table)
                        local b = table[1,1]
                        local se = table[2,1]
                        local pval = table[4, 1]
                        local N = `e(N)'
                        mat coeffs_f`form'_bw`bw' = (`b' \ `se' \ `pval' \ `N')   

                        if "`bw'"=="${test_bw}" & "`form'"=="1" & "`period'" == "post" & "`samp'" == "pooled" {
                            global b_`n'day = `b'
                        }

                        if "`form'" == "1" {
                            local slope_b = table[1,3]
                            local slope_se = table[2,3]
                            local slope_pval = table[4,3]
                            mat slope_f`form'_bw`bw' = ///
                                (`slope_b' \ `slope_se' \ `slope_pval' \ `N')  
                        }
                    } // closes bw loop
                } // closes linear/quadratic loop
                mat rd_`period'_`samp'_`n'day = ///
                    (coeffs_f1_bw3, coeffs_f1_bw5, coeffs_f1_bw10, ///
                     coeffs_f2_bw3, coeffs_f2_bw5, coeffs_f2_bw10)

                mat slp_`period'_`samp'_`n'day = ///
                    (slope_f1_bw3, slope_f1_bw5, slope_f1_bw10)
            } // closes samp loop
        } // closes closes n loop
    } // closes period loop
end

program export_rd_coeffs
    foreach n in 1 30 {
        foreach coef in rd slp {
            mat `coef'_post_`n'day = ///
                `coef'_post_pooled_`n'day \ ///
                `coef'_post_forprof_`n'day \ ///
                `coef'_post_nonprof_`n'day
            matrix_to_txt, saving(../output/tables/`coef'_post_`n'day.txt) ///
                matrix(`coef'_post_`n'day) format(%20.5f) replace ///
                title(<tab:`coef'_post_`n'day>)
        }
        matrix colnames rd_post_`n'day  = lin3 lin5 lin10 quad3 quad5 quad10
        matrix colnames slp_post_`n'day = lin3 lin5 lin10

        foreach coef in rd slp {
            * Export selected coefficients to excel for paper
            clear
            svmat `coef'_post_`n'day, names(col)
            gen varname = ""
            replace varname = "Pooled b"       in 1
            replace varname = "Pooled SE"      in 2
            replace varname = "Pooled p-val"   in 3
            replace varname = "Pooled N"       in 4
            replace varname = "For-prof b"     in 5
            replace varname = "For-prof SE"    in 6
            replace varname = "For-prof p-val" in 7
            replace varname = "For-prof N"     in 8
            replace varname = "Nonprof b"      in 9
            replace varname = "Nonprof SE"     in 10
            replace varname = "Nonprof p-val"  in 11
            replace varname = "Nonprof N"      in 12
            order varname, first
            export excel using "../output/tables/`coef'_post_death_coeffs.xlsx", ///
                firstrow(variables) sheetreplace ///
                sheet(`coef'_post_`n'day)
        }
    }
end

program run_perturbation_test_regs
    foreach period in pre post {
        foreach n in 1 30 {
            use death_`n'day day drgcd using ../external/mort_haz/`period'_panel_pooled, clear

            local bw = ${test_bw}
            local lhlimit = ${min_day} + ${test_bw}
            local rhlimit = ${max_day} - ${test_bw}
            forval t = `lhlimit'/`rhlimit' {    
                if ("`period'"=="pre" & `t'!=0) | ///
                    ("`period'"=="post" & ~inrange(`t', -${test_bw}, ${test_bw})) {
                    * create "fake" day relative to fake threshold
                    gen fday = day - `t'
                    gen right_sso = fday >= 0
                    gen fday_r_sso = fday * right_sso

                    reg death_`n'day right_sso fday fday_r_sso ///
                        if abs(fday)<=`bw', cluster(drgcd) 
                    mat table = r(table)
                    local b = table[1,1]
                    local se = table[2,1]
                    count if abs(fday)<=`bw'
                    local N = `r(N)'
                    mat ft_`period'_`n'day = ///
                        nullmat(ft_`period'_`n'day) \ ///
                            (`t', `b', `se', `N')  
                    drop fday right_sso fday_r_sso
                }
            }
            matrix colnames ft_`period'_`n'day = fake_thresh b se N             
        } // closes n-loop
    } // closes period loop
end

program export_perturb_coeffs
    foreach n in 1 30 {
        clear
        svmat ft_pre_`n'day, names(col)
        save ../temp/ft_pre_`n'day, replace

        clear
        svmat ft_post_`n'day, names(col)
        gen post_pps = 1
        append using ../temp/ft_pre_`n'day
        replace post_pps = 0 if missing(post_pps)
        export excel using ../output/tables/pre_post_fake_coeffs.xlsx, firstrow(variables) ///
            sheet("death_`n'day") sheetreplace
        hist b, bin(20) freq addplot(pci 0 ${b_`n'day} 15 ${b_`n'day}, lcolor(red)) ///
            xtitle("cofficient") scheme(s1mono) legend(order(2 "Post-period Observed")) 
        graph export "../output/figures/perturb_coeffs_pre_post_`n'day.pdf", replace
        compute_pval, n(`n')
    }

    mat coeff_pctiles = pctile_1day, pctile_30day
    matrix_to_txt, saving(../output/tables/coeff_pctiles.txt) matrix(coeff_pctiles) ///
        format(%20.4f) title(<tab:coeff_pctiles>) replace  

    mat coeff_pvals = pval_1day, pval_30day
    matrix_to_txt, saving(../output/tables/coeff_pvals.txt) matrix(coeff_pvals) ///
        format(%20.4f) title(<tab:coeff_pvals>) replace    
end

    program compute_pval
        syntax, n(string)
        local new_N = _N+1
        set obs `new_N'
        gen post_coeff = missing(b)
        replace b = ${b_`n'day} if post_coeff==1
        sort b
        gen cum_pct = _n/_N
        qui sum cum_pct if post_coeff==1
        local pctile = `r(mean)'
        if `pctile' > 0.5 {
            local pval = 2*(1 - `pctile')
        }
        if `pctile' <= 0.5 {
            local pval = 2*(`pctile')
        }
        mat pctile_`n'day = `pctile'
        mat pval_`n'day = `pval'
    end


* EXECUTE
main
