version 14
adopath + ../external/lib/stata/gslab_misc/ado 
adopath + ../external/lib/stata/ado 

preliminaries
set more off
global pre_data "../external/pre_pps/pre_pps_data"
global post_data "../external/compiled/CompiledStayLevelData"

program main
    drgweights
    pre_pay_schedule
    post_pay_schedule
    graph_schedules
    graph_schedules, wt_expr("[aw=drgweight]") suffix("_wt")
    find_moments
    output_moments
    avg_jump
end

program drgweights
    foreach period in pre post {
        use LTCDRG_Code using ../temp/${`period'_data}, clear
        gen num_stays_`period'=1
        gen total_stays_`period' = _N
        collapse (sum) num_stays_`period' (mean) total_stays_`period', by(LTCDRG_Code)
        gen drgweight_`period' = num_stays_`period'/total_stays_`period'
        save ../temp/drgweights_`period', replace
    }
    
    use LTCDRG_Code drgweight_pre using ../temp/drgweights_pre, clear
    merge 1:1 LTCDRG_Code using ../temp/drgweights_post, assert(1 2 3) keep(1 2 3) ///
        keepusing(drgweight_post)
    count if _merge==1
    local pre_nomatch = `r(N)'
    count if _merge==3 | _merge==1
    local pre_nomatch_frac = `pre_nomatch'/`r(N)'
    di "fraction of pre period DRGs not matched to post is `pre_nomatch_frac'"
    egen pre_nomatch_wt = total(drgweight_pre) if _merge==1
    sum pre_nomatch_wt
    di "total of pre-period missing weight is `r(mean)'"
    count if _merge==2
    local post_nomatch = `r(N)'
    count if _merge==3 | _merge==2
    local post_nomatch_frac = `post_nomatch'/`r(N)'
    di "fraction of post period DRGs not matched to pre is `post_nomatch_frac'"
    egen post_nomatch_wt = total(drgweight_post) if _merge==2
    sum post_nomatch_wt
    di "total of post-period missing weight is `r(mean)'"
    drop _merge

    gen drgweight = drgweight_post/drgweight_pre
    replace drgweight = 0 if missing(drgweight)
    keep LTCDRG_Code drgweight
    save ../temp/drgweights, replace
end

program pre_pay_schedule
    use medparid rel_los util_day drgprice Kink dschrgdt LTCDRG_Code ///
        using ${pre_data}, clear
    merge m:1 LTCDRG_Code using ../temp/drgweights, assert(3) keep(3) ///
        keepusing(drgweight) nogen

    gen year = year(dschrgdt)
    merge m:1 year using ../external/cpi/cpi_annual, assert(2 3) keep(3) nogen
    replace drgprice = drgprice/pct_of_12
    * Pre-PPS payment is per diem; find what each stay would be paid on day -15 
    * then do a linear interpolation
    count
    di "# of total pre-period stays (extrapolated to day -15 and day 45) is `r(N)'"
    gen payperday = drgprice / util_day
    gen los_rel_n15 = Kink - 15
    gen drgprice_rel_n15 = los_rel_n15 * payperday
    gen pre_drgprice = drgprice_rel_n15
    save ../temp/pre_payschedule, replace
end

program post_pay_schedule
    use cgs_id medparid cgs_totalpayment cgs_costoutlier LTCDRG_Code cgs_lengthofstay dschrgdt ///
        using "../external_links/CGS Calculate/cgs_output", clear
    merge m:1 medparid using ${post_data}, ///
        assert(1 3) keep(3) nogen keepusing(Kink) // Restrict to baseline
    gen year = year(dschrgdt)
    merge m:1 year using ../external/cpi/cpi_annual, assert(2 3) keep(3) keepusing(pct_of_12) nogen
    gen cgs_drgprice=cgs_totalpayment-cgs_costoutlier
    gen cgs_drgprice_adj = cgs_drgprice/pct_of_12
    rename cgs_lengthofstay util_day
    gen rel_los = util_day - Kink
    save ../temp/merged_post_schedule, replace

    keep if rel_los>= -15 & rel_los <= 45
    unique medparid
    di "# of post-period stays between day -15 and day 45 is `r(sum)'"
    collapse (mean) post_drgprice = cgs_drgprice_adj, by(rel_los)
    * We did not use CGS to compute payments after day 0 since they are the same as on day 0
    * For this reason this dataset contains only "actual stays" rather than all counterfactual
    * LOS for each actual stay after day 0. Therefore to get the average payment schedule
    * we should not take the average for days 1-15 but rather assign the same payment as day 0
    sum post_drgprice if rel_los == 0
    replace post_drgprice = `r(mean)' if rel_los > 0
    save ../temp/post_payschedule, replace
end

program graph_schedules
    syntax, [wt_expr(string)] [suffix(string)] 
    use pre_drgprice payperday drgweight using ../temp/pre_payschedule,clear
    collapse pre_drgprice payperday `wt_expr'
    set obs 61
    gen rel_los = _n - 16
    replace pre_drgprice = pre_drgprice[_n-1] + payperday[1] if rel_los >= -14
    merge 1:1 rel_los using ../temp/post_payschedule, ///
        assert(3) keep(3) keepusing(post_drgprice) nogen
    twoway (connected pre_drgprice rel_los, msymbol(none) lcolor(gs8) lpattern(dash)) ///
        (connected post_drgprice rel_los, msymbol(none) lcolor(black) lpattern(solid)), ///
        xlabel(#7, labsize(small)) ///
        ylabel(0(10000)70000, labsize(small) format(%9.0gc) angle(0))  ///
        yline(0 10000 20000 30000 40000 50000 60000 70000, lcolor(gs15)) ///
        xtitle("Length of Stay Relative to SSO Threshold") ///
        ytitle("Cumulative Payment") scheme(s1mono) legend(order(1 "Pre-PPS" 2 "PPS"))
    graph export ../output/figures/payment_schedule_pre_post`suffix'.pdf, replace
    export_excel_graph, x(rel_los) pre_y(pre_drgprice) post_y(post_drgprice) ///
        sav(payment_schedule_pre_post`suffix')
end

    program export_excel_graph
        syntax, x(string) pre_y(string) post_y(string) sav(string)
        export excel `x' `pre_y' `post_y' using "../output/tables/pre_post_pay_schedules", sheetreplace ///
            firstrow(variables) sheet(`sav') 
    end

program find_moments
    * pre-period payments per day (left of the kink)
    use if rel_los < 0 using ${pre_data}, clear
    merge m:1 year using ../external/cpi/cpi_annual, assert(2 3) keep(3) nogen
    gen payperday_nom = drgprice/util_day
    gen payperday_adj = payperday_nom/pct_of_12
    gen nonprof = AHAcntrl_npngov==1 | AHAcntrl_gov==1
    save ../temp/pre_moment, replace

    use ${post_data}, clear
    merge m:1 year using ../external/cpi/cpi_annual, assert(2 3) keep(3) nogen
    replace calc_jump = calc_jump*1000
    gen payperday_nom = drgprice/util_day
    gen payperday_adj = payperday_nom/pct_of_12
    rename calc_jump calc_jump_nom
    gen calc_jump_adj = calc_jump_nom/pct_of_12
    gen nonprof = AHAcntrl_npngov==1 | AHAcntrl_gov==1
    save ../temp/post_moment, replace
end

program output_moments
    foreach inf in nom adj {
        foreach period in pre post {
            use if rel_los < 0 using ../temp/`period'_moment, clear
            sum payperday_`inf'
            local payperday_`period'_`inf' = r(mean)
            sum payperday_`inf' if AHAcntrl_fp == 1
            local payperday_fp_`period'_`inf' = r(mean)
            sum payperday_`inf' if nonprof == 1
            local payperday_np_`period'_`inf' = r(mean)
        }

        use ../temp/post_moment, clear
        sum calc_jump_`inf'
        local jump_`inf' = r(mean)
        sum calc_jump_`inf' if AHAcntrl_fp == 1
        local jump_fp_`inf' = r(mean)
        sum calc_jump_`inf' if nonprof == 1
        local jump_np_`inf' = r(mean)

        mat moments_`inf' =  ///
            `payperday_pre_`inf'', `payperday_fp_pre_`inf'', `payperday_np_pre_`inf'' \ ///
            `payperday_post_`inf'', `payperday_fp_post_`inf'', `payperday_np_post_`inf'' \ /// 
            `jump_`inf'', `jump_fp_`inf'', `jump_np_`inf''
        matrix_to_txt, saving(../output/tables/moments_`inf'.txt) matrix(moments_`inf') ///
            format(%20.4f) title(<tab:moments_`inf'>) replace
    }
end


program avg_jump
    use medparid rel_los cgs_drgprice_adj if (rel_los == 0) | (rel_los == -1) /// 
        using ../temp/merged_post_schedule, clear
    
    gen type = "low" if rel_los == -1
    replace type = "hi" if rel_los == 0
    drop rel_los

    rename cgs_drgprice_adj drgprice_
    reshape wide drgprice_, i(medparid) j(type) string

    drop if missing(drgprice_low)
    gen sso_jump = drgprice_hi/drgprice_low

    collapse (mean) drgprice_hi drgprice_low jump_avg_of_ratio=sso_jump
    gen jump_ratio_of_avg = drgprice_hi/drgprice_low
    dta_to_txt, saving(../output/tables/sso_jump_ratio.txt) title(<tab:sso_jump_ratio>) ///
        replace noquote nonames
end

* Execute
main
