version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado

preliminaries

program main
    load_data_make_vars
    describe_stats
    run_regs
    iv_regs
    plot_coeffs, var(util_day) varname(Length of Stay)
    plot_coeffs, var(mort30day) varname(30 Day Post-Admission Mortality) yrange("ylab(-.015(.005).015)")
    plot_coeffs, var(mort60day) varname(60 Day Post-Admission Mortality) yrange("ylab(-.015(.005).015)")
    plot_coeffs, var(mort90day) varname(90 Day Post-Admission Mortality) yrange("ylab(-.015(.005).015)")
    n_stay_reg
end

program load_data_make_vars
    use LTCDRG_Code Kink rate_year year using ../external/compiled/CompiledStayLevelData, clear
    collapse (mean) Kink, by(LTCDRG_Code rate_year)
    sort LTCDRG_Code rate_year
    by LTCDRG_Code: gen sso_change = Kink - Kink[_n-1] 
    replace sso_change = 0 if missing(sso_change)
    label variable sso_change "Size of change in SSO threshold since last rate year"
    gen any_year_drg_sso_change = abs(sso_change) > 0
    label variable any_year_drg_sso_change "Any change in SSO threshold since last rate year"
    save ../temp/drg_change_data, replace

    use LTCDRG_Code death_dt admsndt dschrgdt mort60day mort90day util_day rate_year ///
        using ../external/compiled/CompiledStayLevelData, clear
    merge m:1 LTCDRG_Code rate_year using ../temp/drg_change_data, assert(3) keep(3) nogen

    gen mort30day_admsn = (death_dt - admsndt) <= 30
    gen n_stays = 1
    collapse (mean) util_day mort30 mort60 mort90 sso_change any_year_drg_sso_change Kink ///
        (sum) n_stays, by(rate_year LTCDRG_Code)
    label variable any_year_drg_sso_change "Any change in SSO threshold over DRG-year"
    label variable sso_change "Size of change in SSO threshold since last rate year"
    save ../temp/year_drg_data, replace

    bys LTCDRG_Code: egen num_any_changes = total(any_year_drg_sso_change)

    bys LTCDRG_Code: egen max_thres_d = max(Kink)
    bys LTCDRG_Code: egen min_thres_d = min(Kink)
    gen any_drg_sso_change = max_thres_d != min_thres_d
    label variable any_drg_sso_change "Any change in SSO threshold over sample period"
    assert any_drg_sso_change == 0 if num_any_changes == 0

    gen no_drg_sso_change = 1 - any_drg_sso_change
    gen all = 1
    sort LTCDRG_Code rate_year
    save ../temp/data_loaded, replace
end

program describe_stats
    use ../temp/data_loaded, clear
    sum sso_change [aw=n_stays] if sso_change != 0
    local mean_change_cond = `r(mean)'
    gen mt_one_change = num_any_changes > 1 
    gen one_change = num_any_changes == 1
    foreach var in  no_drg_sso_change one_change mt_one_change any_year_drg_sso_change {
        sum `var' [aw=n_stays]
        mat sso_thres_sum_stats = nullmat(sso_thres_sum_stats) \ `r(mean)'
    }
    mat sso_thres_sum_stats = nullmat(sso_thres_sum_stats) \ `mean_change_cond'

    forval year = 2009/2012 {
        sum any_year_drg_sso_change [aw=n_stays] if rate_year == `year'
        mat sso_thres_sum_stats = nullmat(sso_thres_sum_stats) \ `r(mean)'
    }
    matrix_to_txt, saving(../output/tables/sso_thres_sum_stats.txt) matrix(sso_thres_sum_stats) ///
        title(<tab:sso_thres_sum_stats>) format(%20.4f) replace

    clear
    svmat sso_thres_sum_stats
    gen descrp = ""
    replace descrp = "Fraction Stays in DRGs with No SSO Change 2007-2011" in 1
    replace descrp = "Fraction Stays in DRGs with 1 SSO Change 2007-2011" in 2
    replace descrp = "Fraction Stays in DRGs with >1 SSO Change 2007-2011" in 3
    replace descrp = "Fraction Stays in DRGs with an SSO Change within Year" in 4
    replace descrp = "Average Within-Year Change if Any" in 5
    replace descrp = "Fraction Stays in DRGs with an SSO Change from 2008-2009" in 6
    replace descrp = "Fraction Stays in DRGs with an SSO Change from 2009-2010" in 7
    replace descrp = "Fraction Stays in DRGs with an SSO Change from 2010-2011" in 8
    replace descrp = "Fraction Stays in DRGs with an SSO Change from 2011-2012" in 9
    order descrp, first
    export excel ../output/tables/sso_thres_sum_stats.xls, replace
    
end

program run_regs
    use ../temp/data_loaded, clear
    foreach var in util_day mort30day mort60day mort90day {
        areg `var' Kink i.rate_year [aw=n_stays], a(LTCDRG_Code) vce(robust)
        mat coeffs_`var' = _b[Kink] \ _se[Kink]
        sum `var' [aw=n_stays]
        mat means_mat = nullmat(means_mat) \ (`r(mean)', _b[Kink]/`r(mean)')
    }
    mat sso_var_coeffs = coeffs_util_day' \ coeffs_mort30day' \ coeffs_mort60day' \ coeffs_mort90day'
    mat sso_var_coeffs = nullmat(sso_var_coeffs), means_mat
    matrix_to_txt, saving(../output/tables/sso_var_coeffs.txt) matrix(sso_var_coeffs) ///
        title(<tab:sso_var_coeffs>) format(%20.4f) replace
end

program iv_regs
    use ../temp/data_loaded, clear

    set matsize 800
    foreach var in mort30day mort60day mort90day {
        sum `var' [aw=n_stays]
        local `var'_mean = `r(mean)'
        ivregress 2sls `var' i.rate_year i.LTCDRG_Code (util_day = Kink) [aw=n_stays] , vce(robust)
        mat table = r(table)
        mat coeffmat = table[1, 1] , table[2, 1], table[5, 1], table[6, 1]
        mat sso_iv_coeffmat = nullmat(sso_iv_coeffmat) \ ( coeffmat, coeffmat/``var'_mean')
    }
    mat addns = means_mat[2..4, 1..1], sso_var_coeffs[2..4, 1..2]
    mat sso_iv_coeffmat = addns, sso_iv_coeffmat
    matrix_to_txt, saving(../output/tables/sso_iv_coeffmat.txt) matrix(sso_iv_coeffmat) ///
        title(<tab:sso_iv_coeffmat>) format(%20.4f) replace

    matrix colnames sso_iv_coeffmat = varmean rf_coeff rf_coeff_se iv_coeff iv_coeff_se ci95_lo ci95_hi ///
        iv_coeff_pct iv_coeff_pct_se ci95_lo_pct ci95_hi_pct

    clear
    svmat sso_iv_coeffmat, names(col)
    gen descrp = ""
    replace descrp = "30-Day Mortality" in 1
    replace descrp = "60-Day Mortality" in 2
    replace descrp = "90-Day Mortality" in 3
    order descrp, first
    export excel ../output/tables/sso_iv_coeffmat.xls, replace firstrow(variables)
end

program n_stay_reg
    use ../temp/data_loaded, clear
    areg n_stays Kink i.rate_year [aw=n_stays], a(LTCDRG_Code)
    mat coeffs_n_stays = _b[Kink] \ _se[Kink]
    plot_coeffs, var(n_stays) varname(Number of LTCH Stays) yrange("ylab(-2000(1000)2000)")
end

    program plot_coeffs
        syntax, var(string) varname(string) [yrange(string)]

        use ../temp/data_loaded, clear
        local b: di %5.4f coeffs_`var'[1,1]
        local se: di %5.4f coeffs_`var'[2,1]
        sum `var' [aw=n_stays]
        local ymean: di %5.2f `r(mean)'
        sum Kink [aw=n_stays]
        local xmean: di %5.2f `r(mean)'
        xi i.rate_year, pref(_Y)
        binscatter `var' Kink [aw=n_stays], absorb(LTCDRG_Code) controls(_Y*) noaddmean ///
            xtitle("Residualized SSO Threshold") ytitle("Residualized `varname'") ///
            `yrange' note("Slope = `b', SE = `se'") savedata(../output/tables/sso_thres_`var'_data) ///
            title("Mean y var: `ymean'" "Mean x var: `xmean'", size(medsmall) pos(5) ring(0))
        graph export ../output/figures/sso_thres_`var'.pdf, replace
    end

* EXECUTE
main
