version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado
preliminaries
global min_sample_year=1998
global max_sample_year=2014
global min_sample_year_p1 = 1999
global max_sample_year_m1 = 2013

program main
    append_data
    drop_dups
    backfill_fp_status 
    define_colocation 
    define_systems
    impute_ltch
    label_states
    merge_save
end

program append_data
    local keepvars_nonnumeric="sys* mcrnum mname mloczip stcd id"
    local keepvars_numeric="dbegm dbegd dbegy dendm dendd dendy dcov fyr fism fisd lat lon serv mhsmemb resp cntrl"

    forval year=${min_sample_year}/${max_sample_year} {
        use ../external/aha/aha_extract`year', clear
        if `year'<=2007 {
            keep `keepvars_nonnumeric' `keepvars_numeric'
        }
        else if `year'>2007 {
            keep `keepvars_nonnumeric' `keepvars_numeric' lt*
            destring ltchf, replace
            destring ltchc, replace 
            label variable ltchf "Freestanding LTCH"
            label variable ltchc "LTCH collocated within a general ACH"
            label variable ltnm "Name of co-located hospital"
            label variable ltct "City of co-located hospital"
            label variable ltst "State of co-located hospital"
        }
        foreach var in `keepvars_numeric' {
            cap destring `var' ,replace
        }
        gen aha_data_year=`year'
        label variable aha_data_year "AHA Survey Year"

        rename lat latitude
        rename lon longitude
        save ../temp/aha`year', replace
    }

    clear
    forval year = $min_sample_year / $max_sample_year {
        append using ../temp/aha`year'
    }

    rename mcrnum prvdrnum
    drop if missing(prvdrnum)
    drop if prvdrnum == "777777"
    gen provider_id=substr(prvdrnum,3,.)
    destring provider_id, replace force
    gen ach  = inrange(provider_id, 1300, 1399) | ///
               inrange(provider_id, 0, 999)
    gen ltch = inrange(provider_id, 2000, 2299)
    gen snf  = inrange(provider_id, 5000, 6499)
    gen irf  = inrange(provider_id, 3025, 3099)
    gen zipcode= substr(mloczip,1,5)
    gen cntrl_fp=inlist(cntrl,30,31,32,33)
    gen cntrl_npngov=inlist(cntrl,21,22,23)
    gen cntrl_gov=inlist(cntrl,12,13,14,15,16,41,42,43,44,45,46,47,48)
    drop if inlist(stcd,"03","04","05","06","07","08")
    drop if ach + ltch + snf + irf == 0
    assert cntrl_fp + cntrl_npngov + cntrl_gov == 1
    save ../temp/data_collected, replace
end

program drop_dups
    use ../temp/data_collected, clear

    * drop duplicates, keeping the ones with less missing data (dcov is 0 for entries with lots of missing data)
    duplicates tag prvdrnum aha_data_year, gen(dup)
    drop if dup > 0 & dcov==0 
    drop dup

    *some hospitals map to the same provider number, we only keep one entry per prvdrnum but
	*need to make sure that we have all relevant characteristics for this prvdrnum
	*for now we just need sysid, ltchf, and ltchc, but may need others later
	foreach var in sysid ltchf ltchc {
	    gsort prvdrnum aha_data_year -`var'
		replace `var'=`var'[_n-1] if prvdrnum==prvdrnum[_n-1] & ///
                                    aha_data_year==aha_data_year[_n-1] & ///
                                    missing(`var')
	}

    *also need to make sure that the fp/np vars agree
    * (It's okay if one is NPNGOV and the other is GOV, because they'll be combined downstream)
    foreach v in sysid ltchc ltchf cntrl_fp {
        bys prvdrnum aha_data_year: gen `v'_flag = 1 if ///
            (_n==1 & `v'[_n+1] != `v' & prvdrnum[_n+1]==prvdrnum & ///
                                    aha_data_year[_n+1]==aha_data_year) | ///
            (_n==2 & `v'[_n-1] != `v' & prvdrnum[_n-1]==prvdrnum & ///
                                    aha_data_year[_n-1]==aha_data_year)
    }


    * drop all duplicates with conflicting variables
    duplicates tag prvdrnum aha_data_year, gen(dup2)
    tab dup2
    drop if dup2 > 0 & (sysid_flag==1 | cntrl_fp_flag == 1 | ltchc_flag==1 | ltchf_flag == 1)
    drop dup2
    drop *_flag 

    * keep first entry if vars don't conflict
    gsort prvdrnum aha_data_year 
    bys prvdrnum aha_data_year: keep if _n==1

    isid prvdrnum aha_data_year

    *match to HRRs
    merge m:1 zipcode using ../external/xwalk/ziphrrxwalk, assert(1 2 3) keep(1 3) ///
        keepusing(hrrnum) nogen
     
    *the reporting period for year X usually starts in year X-1 and ends in year X
    *which resembles the PSS rate years
    tab dbegy aha_data_year
    tab dendy aha_data_year

    save ../temp/data_long, replace
end

program backfill_fp_status
    use prvdrnum aha_data_year cntrl_fp cntrl_npngov cntrl_gov ach ltch snf irf ///
        using ../temp/data_long, clear
    reshape wide cntrl_fp cntrl_npngov cntrl_gov, i(prvdrnum ach ltch snf irf) ///
        j(aha_data_year)
    foreach v in fp npngov gov {
        egen mean_`v' = rowmean(cntrl_`v'*)
    }

    forval yr = ${min_sample_year}/${max_sample_year} {
        foreach v in fp npngov gov {
            replace cntrl_`v'`yr' = mean_`v' if missing(cntrl_`v'`yr') & (mean_`v'==0 | mean_`v'==1)
        }
    }
    forval yr = ${max_sample_year_m1}(-1)${min_sample_year} {
        foreach v in fp npngov gov {
            local next_yr = `yr'+1
            replace cntrl_`v'`yr' = cntrl_`v'`next_yr' if missing(cntrl_`v'`yr')
        }
    }
    forval yr = ${min_sample_year_p1}(1)${max_sample_year} {
        foreach v in fp npngov gov {
            local past_yr = `yr'-1
            replace cntrl_`v'`yr' = cntrl_`v'`past_yr' if missing(cntrl_`v'`yr')
        }
    }
    reshape long
    assert cntrl_fp + cntrl_gov + cntrl_npngov==1
    keep prvdrnum aha_data_year cntrl_* ach ltch snf irf

    foreach v in fp npngov gov {
        assert ~missing(cntrl_`v')
        rename cntrl_`v' cntrl_`v'bk
    }
    save ../temp/backfilled, replace
end

program define_colocation
   forval year=${min_sample_year}/${max_sample_year} {
        use id longitude latitude aha_data_year provider_id ach ///
            if aha_data_year==`year' & latitude!=. & longitude!=. ///
            & ach == 1 ///
            using ../temp/data_long, clear
        save ../temp/ach_location`year', replace
    } 

   forval year=${min_sample_year}/${max_sample_year} {
        use id longitude latitude aha_data_year provider_id prvdrnum ltch ///
            if aha_data_year==`year' & latitude!=. & longitude!=. ///
            & ltch == 1 ///
            using ../temp/data_long, clear
        save ../temp/LTCH_location`year', replace
    } 

   forval year=${min_sample_year}/${max_sample_year} {
        use id longitude latitude aha_data_year provider_id prvdrnum snf irf ///
            if aha_data_year==`year' & latitude!=. & longitude!=. ///
            & (snf == 1 | irf == 1) ///
            using ../temp/data_long, clear
        save ../temp/SNF_location`year', replace
    } 

   forval year=${min_sample_year}/${max_sample_year} {
        use ../temp/ach_location`year', clear

        foreach v in LTCH SNF {
            geonear id latitude longitude using ../temp/`v'_location`year', ///
                neighbors(id latitude longitude) //result in km
            gen CoLoc`v'_Flag = (km_to_nid<=0.2286) //250 yard=0.2286 km
            rename nid nid_`v'
            rename km_to_nid km_to_nid_`v'
            *gen near_`v' = nid_`v' if CoLoc`v'_Flag == 1
            label variable km_to_nid_`v' "Km to nearest `v'"

            rename id id_ach
            rename nid_`v' id
            merge m:1 id aha_data_year using ../temp/`v'_location`year', ///
                assert(1 2 3) keep(1 3) keepusing(prvdrnum) nogen
            rename prvdrnum nprvdrnum_`v'
            rename id nid_`v'
            rename id_ach id
        }

        save ../temp/colocated`year', replace
    } 

    use ../temp/data_long, clear
    forval year=${min_sample_year}/${max_sample_year} {
        merge 1:1 id aha_data_year using ../temp/colocated`year', ///
            keepusing(CoLocLTCH_Flag CoLocSNF_Flag nprvdrnum_LTCH nprvdrnum_SNF km_to_nid_LTCH km_to_nid_SNF) ///
            update nogen
    }
    assert CoLocLTCH_Flag!=. if latitude!=. & longitude!=. & ach == 1
    assert CoLocSNF_Flag!=. if latitude!=. & longitude!=. & ach == 1

    label variable CoLocLTCH_Flag "LTCH within 250 yds"
    label variable CoLocSNF_Flag "SNF or IRF within 250 yds"
    label variable nprvdrnum_LTCH "Provider number of nearest LTCH neighbor"
    label variable nprvdrnum_SNF "Provider number of nearest SNF neighbor"

    save ../temp/ach_colocated, replace
end

program define_systems
    use ach ltch snf sysid aha_data_year hrrnum zipcode ///
        prvdrnum aha_data_year ///
        using ../temp/data_long if sysid!="", clear
    foreach htype in ach ltch snf {
        bys aha_data_year sysid: egen sys_`htype'=total(`htype')
        bys aha_data_year sysid hrrnum: egen sys_hrr_`htype'=total(`htype')
        bys aha_data_year sysid zipcode: egen sys_zip_`htype'=total(`htype')
        recode sys_`htype' sys_hrr_`htype' sys_zip_`htype' (.=0)
    }
    gen SysACHflag=sys_ach>0
    gen SysHrrACHflag=sys_hrr_ach>0
    gen SysZipACHflag=sys_zip_ach>0

    gen SysLTCHflag=sys_ach>0
    gen SysHrrLTCHflag=sys_hrr_ach>0
    gen SysZipLTCHflag=sys_zip_ach>0

    gen SysSNFflag=sys_ach>0
    gen SysHrrSNFlag=sys_hrr_ach>0
    gen SysZipSNFflag=sys_zip_ach>0

    label variable SysACHflag "Has ACH in System"
    label variable SysHrrACHflag "Has ACH in System within HRR"
    label variable SysZipACHflag "Has ACH in System within zipcode"
    label variable sys_ach "Number of ACHs in System"
    label variable sys_hrr_ach "Number of ACHs in System within HRR"
    label variable sys_zip_ach "Number of ACHs in System wihtin zipcode"

    label variable SysLTCHflag "Has LTCH in System"
    label variable SysHrrLTCHflag "Has LTCH in System within HRR"
    label variable SysZipLTCHflag "Has LTCH in System within zipcode"
    label variable sys_ltch "Number of LTCHs in System"
    label variable sys_hrr_ltch "Number of LTCHs in System within HRR"
    label variable sys_zip_ltch "Number of LTCHs in System wihtin zipcode"

    label variable SysSNFflag "Has SNF in System"
    label variable SysHrrACHflag "Has SNF in System within HRR"
    label variable SysZipACHflag "Has SNF in System within zipcode"
    label variable sys_snf "Number of SNFs in System"
    label variable sys_hrr_snf "Number of SNFs in System within HRR"
    label variable sys_zip_snf "Number of SNFs in System wihtin zipcode"

    save ../temp/system, replace
end

program impute_ltch
    *Create an imputed version of ltchc and ltchf which assumes that ltchc/f status not changed
    *until next reported or before first reported
    use ltchc ltchf prvdrnum aha_data_year using  ../temp/data_long, clear  
    foreach v in ltchc ltchf {
        gen `v'_impute=`v'
        sort prvdrnum aha_data_year
        replace `v'_impute=`v'_impute[_n-1] if prvdrnum==prvdrnum[_n-1] & ///
            `v'_impute[_n-1]!=. & `v'_impute==.
        gsort prvdrnum -aha_data_year
        replace `v'_impute=`v'_impute[_n-1] if prvdrnum==prvdrnum[_n-1] & ///
            `v'_impute[_n-1]!=. & `v'_impute==.
    }
    save ../temp/imputed_ltch_cf, replace
end

program label_states
    * AHA uses a mysterious state numbering system documented in "Appendices" of aha2012_docs
    * which can be found here: http://users.nber.org/~jroth/pdf/
    clear
    set obs 95
    gen stcd = _n
    gen state = ""
    replace state = "ME" if stcd == 11
    replace state = "NH" if stcd == 12
    replace state = "VT" if stcd == 13
    replace state = "MA" if stcd == 14
    replace state = "RI" if stcd == 15
    replace state = "CT" if stcd == 16
    replace state = "NY" if stcd == 21
    replace state = "NJ" if stcd == 22
    replace state = "PA" if stcd == 23
    replace state = "DE" if stcd == 31
    replace state = "MD" if stcd == 32
    replace state = "DC" if stcd == 33
    replace state = "VA" if stcd == 34
    replace state = "WV" if stcd == 35
    replace state = "NC" if stcd == 36
    replace state = "SC" if stcd == 37
    replace state = "GA" if stcd == 38
    replace state = "FL" if stcd == 39
    replace state = "OH" if stcd == 41
    replace state = "IN" if stcd == 42
    replace state = "IL" if stcd == 43
    replace state = "MI" if stcd == 44
    replace state = "WI" if stcd == 45
    replace state = "KY" if stcd == 51
    replace state = "TN" if stcd == 52
    replace state = "AL" if stcd == 53
    replace state = "MS" if stcd == 54
    replace state = "MN" if stcd == 61
    replace state = "IA" if stcd == 62
    replace state = "MO" if stcd == 63
    replace state = "ND" if stcd == 64
    replace state = "SD" if stcd == 65
    replace state = "NE" if stcd == 66
    replace state = "KS" if stcd == 67
    replace state = "AR" if stcd == 71
    replace state = "LA" if stcd == 72
    replace state = "OK" if stcd == 73
    replace state = "TX" if stcd == 74
    replace state = "MT" if stcd == 81
    replace state = "ID" if stcd == 82
    replace state = "WY" if stcd == 83
    replace state = "CO" if stcd == 84
    replace state = "NM" if stcd == 85
    replace state = "AZ" if stcd == 86
    replace state = "UT" if stcd == 87
    replace state = "NV" if stcd == 88
    replace state = "WA" if stcd == 91
    replace state = "OR" if stcd == 92
    replace state = "CA" if stcd == 93
    replace state = "AK" if stcd == 94
    replace state = "HI" if stcd == 95  
    drop if state == ""
    rename state prv_state
    tostring stcd, replace
    save ../temp/state_codes, replace
end

program merge_save
    use ../temp/data_long, clear
    * merge in various new variables
    merge m:1 stcd using ../temp/state_codes, assert(3) keep(3) nogen
    merge 1:1 prvdrnum aha_data_year using ../temp/ach_colocated, assert(1 3) keep(1 3) nogen
    merge 1:1 prvdrnum aha_data_year using ../temp/system, assert(1 3) keep(1 3) nogen
    merge 1:1 prvdrnum aha_data_year using ../temp/imputed_ltch_cf, assert(3) keep(3) nogen ///
        keepusing(ltchc_impute ltchf_impute)
    merge 1:1 prvdrnum aha_data_year using ../temp/backfilled, assert(2 3) keep(2 3) nogen
    foreach v in fp npngov gov {
        replace cntrl_`v' = cntrl_`v'bk if missing(cntrl_`v')
    }
    drop cntrl_fpbk cntrl_npngovbk cntrl_govbk

    *keep only sample years
    keep if aha_data_year<=${max_sample_year} & aha_data_year>=${min_sample_year}
    label variable aha_data_year "AHA Survey Year"
    save_data ../output/aha_append, replace key(prvdrnum aha_data_year)   


    *keep only LTCHs
    use if ltch==1 using ../output/aha_append, clear
    drop CoLoc* nprvdrnum*
    save_data ../output/aha_append_ltch, replace key(prvdrnum aha_data_year)

    *save data for ACH
    use if ach==1 using ../output/aha_append, clear
    save_data ../output/aha_append_ach, replace key(prvdrnum aha_data_year)

    *save data for SNF/IRF
    use if snf==1 | irf==1 using ../output/aha_append, clear
    drop CoLoc* nprvdrnum*
    save_data ../output/aha_append_snf_irf, replace key(prvdrnum aha_data_year)
end

** EXECUTE
main
