version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
preliminaries

global pre_data = "../external/pre_pps/pre_pps_data"
global post_data = "../external/compiled/CompiledStayLevelData"

program main
    make_vars_data
    make_vars_spells
    merge_hospice_hha
    compute_post_ltch_spending_los
    define_dests
    sample_restrict_and_save
end

program make_vars_data
    foreach period in pre post {
        use medparid year admsndt dschrgdt death_dt DestType dstntncd prvdrnum aha_data_year ///
            bene_id drgprice util_day mort30day_dschrg mort60day_admsn rel_los AHA* ///
            using ${`period'_data}, clear
        gen mort60day_dschrg=(death_dt-dschrgdt)<=60    
        gen mort90day_dschrg=(death_dt-dschrgdt)<=90 
        gen nonprof = AHAcntrl_npngov==1 | AHAcntrl_gov==1
        gen forprof = 1 - nonprof
        merge m:1 year using ../external/cpi/cpi_annual, assert(2 3) keep(3) nogen
        replace drgprice = drgprice/pct_of_12
        save ../temp/`period'_data_loaded, replace
    }
end

program make_vars_spells
    foreach period in pre post {
        use ../external_links/post_ltch/post_ltch_spell_`period', clear
        gen not_ltch = 1 - ltch
        * drop first stays that are not LTCH - driven by errors in generating the dataset
        bys bene_spell (admsndt dschrgdt not_ltch): assert ltch==1 if _n==1
        bys bene_spell (admsndt dschrgdt not_ltch): gen stay_seq = _n
        assert stay_seq>1 if ltch==0

        merge m:1 year using ../external/cpi/cpi_annual, assert(2 3) keep(3) nogen
        replace drgprice = drgprice/pct_of_12
        save ../temp/post_ltch_spells_`period', replace
    }
end

program merge_hospice_hha 
    foreach period in pre post {
        use medparid bene_id year bene_spell admsndt dschrgdt ltch dstntncd stay_seq ///
            drgprice util_day ///
            using ../temp/post_ltch_spells_`period', clear

        gen drgprice_lb = drgprice // lb for lower bound
        gen util_day_lb = util_day // no modifications

        gen drgprice_ub = drgprice //  ub for upper bound
        gen util_day_ub = util_day // will be modified

        gen calendar_year = year
        merge m:1 bene_id calendar_year using ../external_links/cost_use/CU_append_`period', ///
            assert(1 2 3) keep(1 3) keepusing(*_mdcr_pmt *_cov_days *_stays hh_visits) nogen
        foreach hosp in acute oip snf hos hh ptd {
            replace `hosp'_mdcr_pmt = 0 if `hosp'_mdcr_pmt==.
        }
        replace hos_cov_days=0 if missing(hos_cov_days)
        gen hha_stay = inlist(dstntncd, 6, 8)
        gen hos_stay = inlist(dstntncd, 50, 51)
        bys bene_spell: egen num_hha_stays = total(hha_stay)
        bys bene_spell: egen num_hos_stays = total(hos_stay)

        save ../temp/post_ltch_merged_`period', replace
    }
end

program compute_post_ltch_spending_los
    * This program computes the post-LTCH spending and LOS for the entire spell of continuous stays
    foreach period in pre post {
        use medparid year bene_id bene_spell drgprice* util_day* admsndt dschrgdt ltch ///
            num_hha_stays num_hos_stays hh_mdcr_pmt hos_mdcr_pmt hos_cov_days stay_seq ///
            using ../temp/post_ltch_merged_`period', clear

        * Recall that bene_spell is the unique identifier for continuous spells of stays,
        * meaning episodes with no break (<=1 day) between stays
        * It might contain sub-episodes of post-LTCH continuous stays if a patient goes back to a ltch
        * at one point during the post-ltch episode
        bys bene_spell: egen num_stays = count(year)
        gen post_ltch_drgprice = 0
        gen post_ltch_util_day = 0
        foreach var in drgprice util_day {
            foreach add in ub lb {
                gen post_ltch_`var'_`add' = 0
            }
        }
        gen num_post_stays = 0

        sort bene_id stay_seq
        * The following loop adds up the total spending and LOS in the subsequent stays
        sum num_stays
        local max_stays = r(max)
        forv i = 1/`max_stays' {
            foreach var in drgprice drgprice_ub drgprice_lb util_day util_day_ub util_day_lb {
                qui replace post_ltch_`var' = post_ltch_`var' + `var'[_n+`i'] ///
                    if bene_spell == bene_spell[_n+`i'] & ///
                    admsndt[_n+`i'] - dschrgdt[_n+`i'-1] <= 1 & ltch == 1
            }
            qui replace num_post_stays = num_post_stays + 1 if bene_spell == bene_spell[_n+`i'] & ///
                ltch == 1           
        }
        assert post_ltch_drgprice == 0 if ltch == 0
        assert post_ltch_util_day == 0 if ltch == 0

        * LB - no modifications
        * UB:
        * Home health
        qui replace post_ltch_drgprice_ub = post_ltch_drgprice + hh_mdcr_pmt ///
            if num_hha_stays > 0 & ltch==1
        qui replace num_post_stays = num_post_stays+1 ///
            if num_hha_stays > 0 & ltch==1
        * Hospice
        qui replace post_ltch_drgprice_ub = post_ltch_drgprice + hos_mdcr_pmt ///
            if num_hos_stays > 0 & ltch==1
        qui replace post_ltch_util_day_ub = post_ltch_util_day + hos_cov_days ///
            if num_hos_stays > 0 & ltch==1
        qui replace num_post_stays = num_post_stays + 1 ///
            if num_hos_stays > 0 & ltch==1
  
        keep medparid year bene_id bene_spell post_ltch_drgprice* post_ltch_util_day* ltch ///
            drgprice
        save ../temp/post_spending_`period'_pps, replace

        * Restrict to the baseline sample
        merge 1:1 medparid year using ../temp/`period'_data_loaded,  ///
            keepusing (medparid year bene_id mort30day_dschrg mort60day_dschrg mort90day_dschrg ///
            mort60day_admsn rel_los dstntncd DestType dschrgdt forprof nonprof) 
        assert inlist(_merge, 1, 3)
        keep if _merge == 3
        drop _merge
        assert ltch == 1 
        save ../temp/post_ltch_spending_los_`period', replace
    }
end

program define_dests
    foreach period in pre post {
        use ../temp/post_ltch_spending_los_`period', clear
        * Define other destinations
        gen DestCombo = 1 if DestType==1 //Home
        replace DestCombo = 2 if inlist(DestType, 2, 4) //ACH, Hospice
        replace DestCombo = 3 if DestType==3 //SNF/IRF
        replace DestCombo = 4 if inlist(DestType,6,7) //LTCH/Other
        label define dest_c 1 "Home/HHA" 2 "ACH/Hospice" 3 "SNF/IRF" 4 "LTCH/Other"
        label values DestCombo dest_c

        * Define low acuity ("downstream") and high acuity ("upstream") destinations
        gen Dest = 1 if inlist(DestType, 1, 3, 6, 7) //Home, SNF/IRF, LTCH, Other
        replace Dest = 2 if inlist(DestType, 2, 4) //ACH, Hospice
        label define dest 1 "Downstream" 2 "Upstream"
        label values Dest dest
        assert DestType == 5 if missing(Dest) // (Expired)
        * Note that there are some destination death patients who have subsequent stays ...
        * mostly just messiness in the data but I didn't want to forget about it
        ooo DestType for those missing Dest
        tab DestType if missing(Dest)
        count if missing(Dest)
        local missing_dest = `r(N)'
        count
        local missing_frac = `missing_dest'/`r(N)'
        disp("Fraction missing upstream/downstream category is: `missing_frac'")

        rename post_ltch_util_day_ub los_ub
        rename post_ltch_util_day_lb los_lb
        rename post_ltch_drgprice_ub price_ub
        rename post_ltch_drgprice_lb price_lb
        rename mort60day_dschrg      mort60 
        rename mort30day_dschrg      mort30
        rename mort90day_dschrg      mort90

        foreach var in price_ub price_lb los_ub los_lb mort60 {
            gen ln_`var' = log(`var'+1)
        }
        save ../temp/post_ltch_spending_los_`period'_dests, replace
    }
end

program sample_restrict_and_save
    foreach period in pre post {
        use ../temp/post_ltch_spending_los_`period'_dests, clear
        * count fraction with LOS < 150 to justify previous restriction of post-period stays 
        * to before July 31 2012
        count if los_ub <=150
        local cnt = `r(N)'
        local frac = `cnt'/_N
        disp("Fraction of `period'-period stays with post-discharge LOS less than 150: `frac'")

        save_data ../output/post_ltch_`period'_period, replace key(medparid year)
    }
end

* EXECUTE
main
    
