/*
carryforward.ado  7-29-2004

David Kantor
Institute for Policy Studies; Johns Hopkins University


Given a series of data in long shape, make an alternative version that
carries-forward values to fill in for missings.

This was originally in ..\psid003\prep264.do (with some hard-codings).

It would be nice to be able to enforce either stability or uniquness in the
sort.  But presently, wee can't.  I've sent messages to Stata about this
(7-21- & 7-29-2004).  For now, we let the calling routine pre-sort it, or
just know that varlist2 (which is presumably used in the invocation) makes
a key with varlist1.

I have asked for a -unique- and/or -stable- option -- as well as access to
varlist2.  As soon as these become availble, use them in the calls to this.

Same for gen_tail, from which some code was borrowed.

4-6-2005: adding -if- and -in-.

*/

*! version 3.2; 4-6-2005
* previous -- version 3; 8-11-2004




program def carryforward, byable(onecall)
version 8.2

syntax varname [if] [in], [gen(name) replace cfindic(name) back carryalong(varlist)]
marksample touse, novarlist

/*
cfindic is an indicator -- that a value is carried-forward (or back).

-back- is just some info affecting for the var label -- in case the actual
direction is backward instead of forward.  It does not control which direction
the carry operation goes; that always goes forward.  But there may be times
that you go backwards by giving it a sort variable that is the negative of
another.  For example,
 gen int negyear = -year
 bysort id (negyear): carryforward m, gen(m2) back
*/


local gen_orig "`gen'"


if "`gen'"=="" & "`replace'" =="" {
 disp as err "you must specify either gen or replace"
 exit 198
}
if "`gen'"~="" & "`replace'" ~="" {
 disp as err "you may not specify both gen and replace"
 exit 198
}

if "`cfindic'" ~= "" {
 confirm new var `cfindic'
 if "`cfindic'" == "`gen'" {
  disp as err "cfindic may not be the same as gen"
  exit 198
 } 
}


local fwdback "fwd"
if "`back'" ~= "" {
 local fwdback "back"
}

local type1: type `varlist'
local lab1 "`varlist'"

/*
lab1 will be a piece of the var label for the new var, if the -gen- option
is used.  Originally we took the var label of varlist if it existed, and the
name otherwise.  Later, I think it is best to just use the name in all cases.
Here is the original code:
 local lab1: var label `varlist'
 if "`lab1'" == "" {
  local lab1 "`varlist'" // var name instead of the label
 }
*/



if "`gen'"~="" {
 confirm new var `gen'
 quietly gen `type1' `gen' = `varlist'
}
else { // must be -replace-
 local gen "`varlist'" // and operate directly in `varlist' under the name gen.

 if "`cfindic'" ~= "" | "`carryalong'" ~= "" {
  tempvar origvalues
  quietly gen `type1' `origvalues' = `varlist'
 }
}

 


if _by() {
 local by "by `_byvars' :"
}

`by' replace `gen' = `gen'[_n-1] if `touse' & _n>1 & mi(`gen') & ~mi(`gen'[_n-1])


if "`cfindic'" == "" & "`carryalong'" ~= "" {
 /* Need cfindic, but make it a temp */
 tempvar cfindic
}

if "`cfindic'" ~= "" {
 if "`gen_orig'"~="" {
  gen byte `cfindic' = `gen' ~= `varlist'
 }
 else { // must be -replace-
  gen byte `cfindic' = `gen' ~= `origvalues'
 }

 label var `cfindic' "`gen' has a carry-`fwdback' value"
 assert ~mi(`gen') if `cfindic'
}


if "`gen_orig'"~="" {
 label var `gen' "`lab1'; posbly carrd `fwdback'"
}
else { // must be -replace-
 notes `varlist': possibly carried `fwdback'
}

/* That is, if we generate a new item, it's label gets the message about it
being possibly carried forward (or back).  But if we replace, then that
message goes into a note instead.
*/


if "`carryalong'" ~= "" {
 tempvar t002
 foreach var of local carryalong {
  local type2: type `var'

  quietly gen `type2' `t002' = `var'
  quietly `by' replace `var' = `var'[_n-1] if _n>1 & `cfindic'
  quietly count if `var' ~= `t002'
  local nchanges = r(N)
  disp as txt "(`var': `nchanges' changes made)"
  notes `var': possibly carried `fwdback'
  drop `t002'
 }
}

end // carryforward
