function [output, cfRetStruct_out] = calculate_counterfactual_payments(...
    scenarioID, dischargePat, cfParamsStruct_arg, cfRetStruct_arg)


nonltchPayModel = cfParamsStruct_arg.nonltchPayModel;

% Initialize the output container.
cfRetStruct_out = cfRetStruct_arg;

% Define required variables that are local to this function.
% days_cf = cfParamsStruct_arg.days_cf;
T_cf = cfParamsStruct_arg.T_cf;
TT_cf = cfParamsStruct_arg.TT_cf;
p_cf = cfRetStruct_arg.(['p_' scenarioID]);
params_all_hat      = cfRetStruct_arg.params_all_hat;
paramsEstimated     = cfRetStruct_arg.paramsEstimated;
gamma               = 1e3*params_all_hat(strcmp(paramsEstimated,'gamma'));
repCostPerDay       = cfParamsStruct_arg.repCostPerDay_post;
z                   = cfRetStruct_arg.z_post;

% Share of patients admitted to LTCH discharged to ACH/SNF.
% We have two versions based on dischargePat (discharge pattern):
% counterfactual discharge (Cfd) pattern and baseline discharge (Bld) pattern.
if strcmp(dischargePat, 'Cfd')
    % Discharge pattern varies by counterfactual payment schedules.
    d_share_l_cf = cfRetStruct_arg.(['d_share_l_post_' scenarioID]);
    share_a_cf = cfRetStruct_arg.(['share_a_post_' scenarioID]);
    share_s_cf = cfRetStruct_arg.(['share_s_post_' scenarioID]);
    share_m_cf = cfRetStruct_arg.(['share_m_post_' scenarioID]);
    
    % For the linear model, we need the the average health of discharged
    % patients each day.
    avg_health_a_cf = cfRetStruct_arg.(['h_a_post_' scenarioID]);   
    avg_health_s_cf = cfRetStruct_arg.(['h_s_post_' scenarioID]);
    
    % For the non-linear model, we need the the health distribution of
    % discharged patients each day.
    healthDist_a_cf = cfRetStruct_arg.(['h_a_dist_post_' scenarioID]);   
    healthDist_s_cf = cfRetStruct_arg.(['h_s_dist_post_' scenarioID]);
    
    % Convert percentage never discharged from LTCH to decimals.
    share_l_never_disc = cfRetStruct_arg.(['pct_l_never_disc_post_' scenarioID]) / 100;    
elseif strcmp(dischargePat, 'Bld')
    % Discharge pattern is always from the baseline payment schedule.
    d_share_l_cf = cfRetStruct_arg.d_share_l_post_baseline;
    share_a_cf   = cfRetStruct_arg.share_a_post_baseline;
    share_s_cf   = cfRetStruct_arg.share_s_post_baseline;
    share_m_cf   = cfRetStruct_arg.share_m_post_baseline;
    
    % For the linear model, we need the the average health of discharged
    % patients each day.
    avg_health_a_cf = cfRetStruct_arg.h_a_post_baseline;   
    avg_health_s_cf = cfRetStruct_arg.h_s_post_baseline;
    
    % For the non-linear model, we need the the health distribution of
    % discharged patients each day.
    healthDist_a_cf = cfRetStruct_arg.h_a_dist_post_baseline;   
    healthDist_s_cf = cfRetStruct_arg.h_s_dist_post_baseline;
    
    % Convert percentage never discharged from LTCH to decimals.
    share_l_never_disc = cfRetStruct_arg.pct_l_never_disc_post_baseline / 100;
else
    fprintf('Error: ''%s'' is not a valid argument for dischargePat.\n', dischargePat);
    fprintf(['Acceptable options are ''Cfd'' (counterfactual discharges) ' ...
        'and ''Bld'' (baseline discharges).\n']);
    return
end

% Average health of (marginal) patients discharged.
avg_health_a = share_a_cf' * avg_health_a_cf / sum(share_a_cf);
avg_health_s = share_s_cf' * avg_health_s_cf / sum(share_s_cf);

% Total share of people dischrged to different destinations.
tot_share_a = sum(share_a_cf);
tot_share_s = sum(share_s_cf);
tot_share_m = sum(share_m_cf);

% Extract the coefficients for post-LTCH payments and days models.
zeta_a = cfRetStruct_arg.(['zeta_a_post_', nonltchPayModel]);
zeta_s = cfRetStruct_arg.(['zeta_s_post_', nonltchPayModel]);
coefDays_a = cfRetStruct_arg.(['coefDays_a_post_', nonltchPayModel]);
coefDays_s = cfRetStruct_arg.(['coefDays_s_post_', nonltchPayModel]);

% Post-LTCH payments and days to ACH/SNF differ by the payment model we choose.
% Note that regardless of whether our estimation aggregates the post-LTCH
% quantities by pre/post-SSO or not, our prediction here always varies by
% day!
nonltchPay_a_hat = nonltch_spec(nonltchPayModel, zeta_a,     z, healthDist_a_cf);
nonltchPay_s_hat = nonltch_spec(nonltchPayModel, zeta_s,     z, healthDist_s_cf);
nonltchDays_a_hat = nonltch_spec(nonltchPayModel, coefDays_a, z, healthDist_a_cf);
nonltchDays_s_hat = nonltch_spec(nonltchPayModel, coefDays_s, z, healthDist_s_cf);


% Estimated post-LTCH payments/days for ACH, total. Check for negative payments.
tot_payments_a = sum(share_a_cf .* max(nonltchPay_a_hat,0));
temp_tot_payments_a = sum(share_a_cf .* nonltchPay_a_hat);
    temp_diff = tot_payments_a - temp_tot_payments_a;
    if abs(temp_diff) >= 10e-2
        fprintf('ACH: Flooring daily payments at zero makes a difference.\n');
        fprintf('Floor - No Floor = %.2f.\n', temp_diff);
    end
% Average post-LTCH days for patients discharge to ACH. Need to re-weight
% by the total share of patients discharged there.
avg_days_a = sum(share_a_cf .* max(nonltchDays_a_hat,0)) / tot_share_a;

% Estimated post-LTCH payments/days for SNF, total. Check for negative payments.
tot_payments_s = sum(share_s_cf .* max(nonltchPay_s_hat,0));
temp_tot_payments_s = sum(share_s_cf .* nonltchPay_s_hat);
    temp_diff = tot_payments_s - temp_tot_payments_s;
    if abs(temp_diff) >= 10e-2
        fprintf('SNF: Flooring daily payments at zero makes a difference.\n');
        fprintf('Floor - No Floor = %.2f.\n', temp_diff);
    end
% Average post-LTCH days for patients discharge to SNF. Need to re-weight
% by the total share of patients discharged there.
avg_days_s = sum(share_s_cf .* max(nonltchDays_s_hat,0));

% Average length of stay at LTCH, payments to LTCH.
alos = share_l_never_disc * TT_cf + sum(d_share_l_cf .* (1:1:TT_cf)');
payments_l = share_l_never_disc*p_cf(TT_cf) + sum(d_share_l_cf.*p_cf);
payments_l_per_day = payments_l/alos;
profits_l = payments_l - alos * gamma * repCostPerDay;
profits_l_per_day = profits_l/alos;
% Share not discharged from LTCH after 60 days.
share_l_not_disc_60days = 1-sum(d_share_l_cf(1:60));
    
% Average payments to ACH/SNF.
avg_payments_a = tot_payments_a / tot_share_a;
avg_payments_s = tot_payments_s / tot_share_s;

% System-wide Medicare payments.
tot_payments = payments_l + tot_payments_a + tot_payments_s;

output = [share_l_not_disc_60days; share_l_never_disc; ...
    tot_payments; ...
    payments_l; profits_l; alos; payments_l_per_day; profits_l_per_day; ...
    tot_payments_a; tot_share_a; avg_payments_a; avg_days_a; avg_health_a; ...
    tot_payments_s; tot_share_s; avg_payments_s; avg_days_s; avg_health_s; ...
    tot_share_m];
clear temp_*;


%%% Split important quantities by pre/post-SSO
output_addtl = [];
for destID = {'a','s','m'}
    destID = cell2mat(destID);
    
    temp_share = eval(['share_' destID '_cf']);
    
    temp_tot_share_preSSO = sum(temp_share(1:T_cf-1)); 
    temp_tot_share_postSSO = sum(temp_share(T_cf:TT_cf));
    
    % Shares discharged pre and post-SSO, reweight so that each vector sums
    % to 1
    temp_share_preSSO  = temp_share(1:T_cf-1)/temp_tot_share_preSSO;
    temp_share_postSSO = temp_share(T_cf:TT_cf)/temp_tot_share_postSSO;
    
    % Share discharged to a specific destination.
    eval(['tot_share_' destID '_preSSO  = temp_tot_share_preSSO;']);
    eval(['tot_share_' destID '_postSSO = temp_tot_share_postSSO;']);
    
    output_addtl = [output_addtl '; tot_share_' destID '_preSSO' '; tot_share_' destID '_postSSO'];
    
    % The following are not computed for "m (death)"
    if ~strcmp(destID, 'm')
        % Average health
        eval(['avg_health_' destID '_preSSO = sum(temp_share_preSSO .* avg_health_', ...
            destID '_cf(1:T_cf-1));']);
        eval(['avg_health_' destID '_postSSO = sum(temp_share_postSSO .* avg_health_', ...
            destID '_cf(T_cf:TT_cf));']);
        output_addtl = [output_addtl '; avg_health_' destID '_preSSO' ...
            '; avg_health_' destID '_postSSO'];
        
        % Total payments to that facility
        eval(['tot_payments_' destID '_preSSO = sum(temp_share(1:T_cf-1) .* nonltchPay_' ...
            destID '_hat(1:T_cf-1));']);
        eval(['tot_payments_' destID '_postSSO = sum(temp_share(T_cf:TT_cf) .* nonltchPay_' ...
            destID '_hat(T_cf:TT_cf));']);
        output_addtl = [output_addtl '; tot_payments_' destID '_preSSO' ...
            '; tot_payments_' destID '_postSSO'];
        
        % Payments per discharge
        eval(['avg_payments_' destID '_preSSO = tot_payments_' destID '_preSSO/temp_tot_share_preSSO;']);
        eval(['avg_payments_' destID '_postSSO = tot_payments_' destID '_postSSO/temp_tot_share_postSSO;']);
        output_addtl = [output_addtl '; avg_payments_' destID '_preSSO' ...
            '; avg_payments_' destID '_postSSO'];
        
        % Average post-LTCH days
        eval(['avg_days_' destID '_preSSO = sum(temp_share_preSSO .* nonltchDays_' ...
            destID '_hat(1:T_cf-1));']);
        eval(['avg_days_' destID '_postSSO = sum(temp_share_postSSO .* nonltchDays_' ...
            destID '_hat(T_cf:TT_cf));']);
        output_addtl = [output_addtl '; avg_days_' destID '_preSSO' ...
            '; avg_days_' destID '_postSSO'];
    end
end

clear temp_*;
output = eval(['[output;' output_addtl ']']);


% Populate cfRetStruct_out with output to return.
if strcmpi(dischargePat, 'Cfd')
    fldTag = [scenarioID '_' nonltchPayModel];
    cfRetStruct_out.(['nonltchPay_a_post_hat_',fldTag])= nonltchPay_a_hat;
    cfRetStruct_out.(['nonltchPay_s_post_hat_',fldTag])= nonltchPay_s_hat;
    cfRetStruct_out.(['nonltchDays_a_post_hat_',fldTag])= nonltchDays_a_hat;
    cfRetStruct_out.(['nonltchDays_s_post_hat_',fldTag])= nonltchDays_s_hat;
end

if strcmpi(scenarioID, 'baseline')
    cfRetStruct_out.(['payments_l_post_', scenarioID])= payments_l;
    cfRetStruct_out.(['payments_l_per_day_post_' scenarioID])= payments_l_per_day;
end