%%  Create counterfactual payment schedules 
% schedule vary by scenario and time horizon
% The major outputs (in a struct):
% p_baseline p_cons p_consCap p_lower p_higher
function [cfRetStruct_out, cf_scenarios] = ...
    create_counterfactual_payment_schedules(...
    cfParamsStruct, retStruct_arg, cf_scenarios)


    % We force a cap for these scenarios: sPayCap and estPayCap.
    sPayCapDay = 60;
    estCostCapDay = 60;
    period = 'post';

    % Initialize the structure to return as an output.
    cfRetStruct_out = retStruct_arg;

    % Local variables from structs.
    backInductT_cf = cfParamsStruct.backInductT_cf;
    T_cf    = cfParamsStruct.T_cf;
    TT_cf   = cfParamsStruct.TT_cf;
    days_cf = cfParamsStruct.days_cf;

    p_preSSO_perdiem    = cfParamsStruct.(['p_perdiem_', period]);
    p_postSSO           = cfParamsStruct.(['p_postSSO_', period]);
    p_on_T              = cfParamsStruct.(['p_preSSO_', period])(T_cf);

    params_all_hat      = retStruct_arg.params_all_hat;
    paramsEstimated     = retStruct_arg.paramsEstimated;
    gamma               = 1e3*params_all_hat(strcmp(paramsEstimated,'gamma'));
    estCostPerDay       = gamma * cfParamsStruct.(['repCostPerDay_', period]);    
    sPayPerDay          = cfParamsStruct.(['sPayPerDay_', period]);

    %%%%%%%%%%%%%%%%%%
    %COUNTERFACTUALS
    %%%%%%%%%%%%%%%%%
    % Baseline
    p_postSSO_baseline = p_postSSO;
    p_preSSO_baseline  = p_preSSO_perdiem*ones(T_cf,1);
    p_preSSO_baseline(T_cf) = p_on_T;

    % Constant forever
    p_postSSO_cons = p_preSSO_perdiem;
    p_preSSO_cons  = p_preSSO_perdiem*ones(backInductT_cf,1);

    % Constant with cap at DRG level; extended SSO.
    cap = sum(p_preSSO_baseline);
    T_consCap = floor(cap/p_preSSO_perdiem)+1;
    cfRetStruct_out.T_consCap = T_consCap;
    remainder = mod(cap,p_preSSO_perdiem);
    p_postSSO_consCap = 0;
    p_preSSO_consCap = p_preSSO_perdiem*ones(T_consCap-1,1);
    p_preSSO_consCap(T_consCap,1) = remainder;

    % No jump (reduce DRG)
    p_postSSO_lower = 0;
    p_preSSO_lower  = p_preSSO_perdiem*ones(T_cf,1);

    % No jump (higher per-diem)
    p_postSSO_higher = 0;
    p_preSSO_higher  = ((T_cf-1)/T_cf*p_preSSO_perdiem + p_on_T/T_cf)*ones(T_cf,1);

    % Linear forever: pay LTCH SNF's amount.
    p_postSSO_sPay = sPayPerDay;
    p_preSSO_sPay  = sPayPerDay*ones(T_cf,1);

    % Linear with cap: pay LTCH SNF's amount in the per-diem part.
    p_postSSO_sPayCap = 0;
    p_preSSO_sPayCap  = sPayPerDay*ones(sPayCapDay,1);

    % Linear forever: pay LTCH its estimated cost.
    p_postSSO_estCost = estCostPerDay;
    p_preSSO_estCost  = estCostPerDay * ones(T_cf,1);

    % Linear with cap: pay LTCH its estimated cost in the per-diem part.
    p_postSSO_estCostCap = 0;
    p_preSSO_estCostCap  = estCostPerDay * ones(estCostCapDay,1);

    % Add a new column to counterfactual_scenarios to indicate T (new "SSO")
    % for each scenario. Exceptions:
    % consCap scenario has T_consCap instead of T.
    % estPayCap and sPayCap have 60 instead of T.
    cf_scenarios.T = T_cf*ones(numel(cf_scenarios.scenarioID),1);
    cf_scenarios('consCap','T') = {T_consCap};
    cf_scenarios('sPayCap','T') = {sPayCapDay};
    cf_scenarios('estCostCap','T') = {estCostCapDay};

    % Initialize counterfactual payment schedules for all periods.
    p_baseline = zeros(TT_cf,1);
    p_cons = zeros(TT_cf,1);
    p_consCap = zeros(TT_cf,1);
    p_higher = zeros(TT_cf,1);
    p_lower = zeros(TT_cf,1);
    p_sPay = zeros(TT_cf,1);
    p_estCost = zeros(TT_cf,1);

    % Period 1
    p_baseline(1) = p_preSSO_baseline(1);
    p_cons(1) = p_preSSO_cons(1);
    p_consCap(1) = p_preSSO_consCap(1);
    p_lower(1) = p_preSSO_lower(1);
    p_higher(1) = p_preSSO_higher(1);
    p_sPay(1) = p_preSSO_sPay(1);
    p_estCost(1) = p_preSSO_estCost(1);

    % Subsequent periods.
    for t = 2:T_cf
        p_baseline(t) = p_baseline(t-1) + p_preSSO_baseline(t);

        p_cons(t) = p_cons(t-1) + p_preSSO_cons(t);    
        p_lower(t) = p_lower(t-1) + p_preSSO_lower(t);
        p_higher(t) = p_higher(t-1) + p_preSSO_higher(t);
        p_sPay(t) = p_sPay(t-1) + p_preSSO_sPay(t);
        p_estCost(t) = p_estCost(t-1) + p_preSSO_estCost(t);
    end
    for t = T_cf+1:TT_cf
        p_baseline(t) = p_baseline(t-1) + p_postSSO_baseline;

        p_cons(t) = p_cons(t-1) + p_postSSO_cons;    
        p_lower(t) = p_lower(t-1) + p_postSSO_lower;
        p_higher(t) = p_higher(t-1) + p_postSSO_higher;
        p_sPay(t) = p_sPay(t-1) + p_postSSO_sPay;
        p_estCost(t) = p_estCost(t-1) + p_postSSO_estCost;
    end

    % This is for the consCap scenario only, since it has a different
    % counterfactual "SSO threshold" date.
    for t = 2:T_consCap
        p_consCap(t) = p_consCap(t-1) + p_preSSO_consCap(t);
    end
    for t = T_consCap+1:TT_cf
        p_consCap(t) = p_consCap(t-1) + p_postSSO_consCap;
    end

    p_sPayCap = sPayPerDay * [(1:sPayCapDay)'; sPayCapDay * ones(TT_cf-sPayCapDay,1)];
    p_estCostCap = estCostPerDay * [(1:estCostCapDay)'; estCostCapDay * ones(TT_cf-estCostCapDay,1)];

    % Put payment schedules into the cfRetStruct_out container.
    cfRetStruct_out.p_preSSO_baseline = p_preSSO_baseline;
    cfRetStruct_out.p_postSSO_baseline = p_postSSO_baseline;
    cfRetStruct_out.p_baseline = p_baseline;

    % These counterfactual payment schedules only apply to the post-PPS period.
    cfRetStruct_out.p_preSSO_cons       = p_preSSO_cons;
    cfRetStruct_out.p_preSSO_consCap    = p_preSSO_consCap;
    cfRetStruct_out.p_preSSO_lower      = p_preSSO_lower;
    cfRetStruct_out.p_preSSO_higher     = p_preSSO_higher;
    cfRetStruct_out.p_preSSO_sPay       = p_preSSO_sPay;
    cfRetStruct_out.p_preSSO_estCost    = p_preSSO_estCost;
    cfRetStruct_out.p_preSSO_sPayCap    = p_preSSO_sPayCap;
    cfRetStruct_out.p_preSSO_estCostCap = p_preSSO_estCostCap;

    cfRetStruct_out.p_postSSO_cons       = p_postSSO_cons;
    cfRetStruct_out.p_postSSO_consCap    = p_postSSO_consCap;
    cfRetStruct_out.p_postSSO_lower      = p_postSSO_lower;
    cfRetStruct_out.p_postSSO_higher     = p_postSSO_higher;
    cfRetStruct_out.p_postSSO_sPay       = p_postSSO_sPay;
    cfRetStruct_out.p_postSSO_estCost    = p_postSSO_estCost;
    cfRetStruct_out.p_postSSO_sPayCap    = p_postSSO_sPayCap;
    cfRetStruct_out.p_postSSO_estCostCap = p_postSSO_estCostCap;

    cfRetStruct_out.p_cons       = p_cons;
    cfRetStruct_out.p_consCap    = p_consCap;
    cfRetStruct_out.p_lower      = p_lower;
    cfRetStruct_out.p_higher     = p_higher;
    cfRetStruct_out.p_sPay       = p_sPay;
    cfRetStruct_out.p_estCost    = p_estCost;
    cfRetStruct_out.p_sPayCap    = p_sPayCap;
    cfRetStruct_out.p_estCostCap = p_estCostCap;

    % Export payment schedules to Excel.
    if strcmpi(cfParamsStruct.version, 'v18_a3_pool')
        xlsOutFile = cfParamsStruct.xlsOutFile;
        xlsSheetName = ['payment_schedules_' cfParamsStruct.xlsShtNm];

        if strcmp(period,'pre')
            temp_colnames = {'days' 'p_baseline'};
            temp_outmat = [days_cf' p_baseline];
        elseif strcmp(period,'post')
            temp_colnames = {'days'};
            temp_outmat = days_cf';
            includedScenarios = cf_scenarios.scenarioID(cf_scenarios.include==1);
            for scenarioID = {includedScenarios{:}}
                scenarioID = cell2mat(scenarioID);
                temp_name = ['p_' scenarioID];
                temp_column = eval(['p_' scenarioID]);

                temp_colnames = [temp_colnames {temp_name}];
                temp_outmat = [temp_outmat temp_column];
            end
        end

        %write counterfactual schedule
        xlswrite(xlsOutFile,temp_colnames,xlsSheetName);
        xlswrite(xlsOutFile,temp_outmat,xlsSheetName,'A2');

        %print time stamp
        print_timestamp(xlsOutFile, xlsSheetName, cfParamsStruct.version, cfParamsStruct.run);
    end
end