function [sse_hat, paramsStruct] = objective_function(params_all, moments, W_0, paramsStruct)

    %% estimation    
    [moments_hat] = collect_obj_results(params_all, paramsStruct);
    
    %% SSE
    sse_hat = calc_sse(moments, moments_hat, W_0, params_all, paramsStruct);

    % Add moments_hat: we need this for the Jacobian.
    paramsStruct.moments_hat_jacobian = moments_hat;

end


%% calculate SSE
function sse_hat = calc_sse(moments, moments_hat, W_0, params_all, paramsStruct)
%% SSE
    e = moments - moments_hat;

    % Only multiply the discharge shares by 100.
    % In the fullest model, we stack the moments as such:
    % 1. pre-PPS discharges (length TT)
    % 2. post-PPS discharges (length TT)
    % 3. pre-PPS post-LTCH payments (length TT or 2 when agg by pre/post-SSO)
    % 4. post-PPS post-LTCH payments (length TT or 2 when agg by pre/post-SSO)
    e_adj = e;
    

    switch(paramsStruct.momentsUsed)
        case('pre_post')
            temp_endIndex = 6*paramsStruct.TT;
        case {'pre','post'}
            temp_endIndex = 3*paramsStruct.TT;    
    end
    e_adj(1:temp_endIndex) = e(1:temp_endIndex)*100;
    sse_hat = e_adj' * W_0 * e_adj;
    
    %regularize certain parameters 
    sse_hat = params_regularize(sse_hat, params_all, paramsStruct);


end

%% regularize some parameters
function sse_hat = params_regularize(sse_hat, params_all, paramsStruct)
    

    % parameter penalty
    % Parameters to penalize the objective if certain parameter values violate
    % some constraints.
    penalty_rho     = paramsStruct.penalty_rho;
    penalty_sigma   = paramsStruct.penalty_sigma;
    backInductT     = paramsStruct.backInductT; 

    % get health parameters we want to restrict
    rhoPre    = params_all(strcmp(paramsStruct.paramsEstimated,'rhoPre'));
    rho1Pre   = params_all(strcmp(paramsStruct.paramsEstimated,'rho1Pre'));
    sigmaPre  = params_all(strcmp(paramsStruct.paramsEstimated,'sigmaPre'));
    rhoPost   = params_all(strcmp(paramsStruct.paramsEstimated,'rhoPost'));
    rho1Post  = params_all(strcmp(paramsStruct.paramsEstimated,'rho1Post'));
    sigmaPost = params_all(strcmp(paramsStruct.paramsEstimated,'sigmaPost'));
        
    
    % Impose penalties to constrain parameters.
    if sigmaPre < 1e-5
        sse_hat = sse_hat + penalty_sigma;
    end
    if sigmaPost < 1e-5
        sse_hat = sse_hat + penalty_sigma;
    end


    % We need a different condition for a5: rho is time-varying.
    switch(paramsStruct.estType)
        case('a5')
            % In the expanded rho model (health gets more persistent with t),
            % the max abs(rho + rho1*ln(t)) happens either on backInductT (we force
            % the health process to be stationary after backInductT) or t=1.
            if max(abs(rhoPre),abs(rhoPre + rho1Pre*log(backInductT))) > 0.99999
                sse_hat = sse_hat + penalty_rho;
            end
            if max(abs(rhoPost),abs(rhoPost + rho1Post*log(backInductT))) > 0.99999
                sse_hat = sse_hat + penalty_rho;
            end
        otherwise
            if abs(rhoPre) > 0.99999
                sse_hat = sse_hat + penalty_rho;
            end
            if abs(rhoPost) > 0.99999
                sse_hat = sse_hat + penalty_rho;
            end
    end
end