function [x,fval,exitflag,output] = par_fminsearch(funfcn,x,options,maxlab,varargin)
%FMINSEARCH Multidimensional unconstrained nonlinear minimization (Nelder-Mead).
%   X = FMINSEARCH(FUN,X0) starts at X0 and attempts to find a local minimizer 
%   X of the function FUN.  FUN is a function handle.  FUN accepts input X and 
%   returns a scalar function value F evaluated at X. X0 can be a scalar, vector 
%   or matrix.
%
%   X = FMINSEARCH(FUN,X0,OPTIONS)  minimizes with the default optimization
%   parameters replaced by values in the structure OPTIONS, created
%   with the OPTIMSET function.  See OPTIMSET for details.  FMINSEARCH uses
%   these options: Display, TolX, TolFun, MaxFunEvals, MaxIter, FunValCheck,
%   PlotFcns, and OutputFcn.
%
%   X = FMINSEARCH(PROBLEM) finds the minimum for PROBLEM. PROBLEM is a
%   structure with the function FUN in PROBLEM.objective, the start point
%   in PROBLEM.x0, the options structure in PROBLEM.options, and solver
%   name 'fminsearch' in PROBLEM.solver. 
%
%   [X,FVAL]= FMINSEARCH(...) returns the value of the objective function,
%   described in FUN, at X.
%
%   [X,FVAL,EXITFLAG] = FMINSEARCH(...) returns an EXITFLAG that describes
%   the exit condition. Possible values of EXITFLAG and the corresponding
%   exit conditions are
%
%    1  Maximum coordinate difference between current best point and other
%       points in simplex is less than or equal to TolX, and corresponding 
%       difference in function values is less than or equal to TolFun.
%    0  Maximum number of function evaluations or iterations reached.
%   -1  Algorithm terminated by the output function.
%
%   [X,FVAL,EXITFLAG,OUTPUT] = FMINSEARCH(...) returns a structure
%   OUTPUT with the number of iterations taken in OUTPUT.iterations, the
%   number of function evaluations in OUTPUT.funcCount, the algorithm name 
%   in OUTPUT.algorithm, and the exit message in OUTPUT.message.
%
%   Examples
%     FUN can be specified using @:
%        X = fminsearch(@sin,3)
%     finds a minimum of the SIN function near 3.
%     In this case, SIN is a function that returns a scalar function value
%     SIN evaluated at X.
%
%     FUN can be an anonymous function:
%        X = fminsearch(@(x) norm(x),[1;2;3])
%     returns a point near the minimizer [0;0;0].
%
%     FUN can be a parameterized function. Use an anonymous function to
%     capture the problem-dependent parameters:
%        f = @(x,c) x(1).^2+c.*x(2).^2;  % The parameterized function.
%        c = 1.5;                        % The parameter.
%        X = fminsearch(@(x) f(x,c),[0.3;1])
%        
%   FMINSEARCH uses the Nelder-Mead simplex (direct search) method.
%
%   See also OPTIMSET, FMINBND, FUNCTION_HANDLE.

%   Reference: Jeffrey C. Lagarias, James A. Reeds, Margaret H. Wright,
%   Paul E. Wright, "Convergence Properties of the Nelder-Mead Simplex
%   Method in Low Dimensions", SIAM Journal of Optimization, 9(1):
%   p.112-147, 1998.

%   Copyright 1984-2015 The MathWorks, Inc.

%%
%----- YWS: big picture: additional params for par_fminsearch -----
% numWorkers to use - if not passed in, use default (i.e. max of max - some_num)

%%
%----- YWS: Options that should have been passed in -----
% Comment this whole section out when calling par_fminsearch
% Set up
% numParams = 10; % should be determined programatically
% opt_MaxFunEvals = 1e2;  % part of the options input arg
% opt_MaxIter = 1e2;      % part of the options input arg
% opt_TolFun = 1e-8;      % part of the options input arg
% opt_TolX = 1e-8;        % part of the options input arg
% 
% % Input arguments
% funfcn = @leewisFnEg1;
% x = 1:numParams;    % I call this x because it's called as such by fiminsearch
% options = optimset('Disp', 'iter', 'MaxFunEvals', opt_MaxFunEvals, ...
%     'MaxIter', opt_MaxIter, 'TolFun', opt_TolFun, 'TolX', opt_TolX);
% varargin = {};

%%
defaultopt = struct('Display','notify','MaxIter','200*numberOfVariables',...
    'MaxFunEvals','200*numberOfVariables','TolX',1e-4,'TolFun',1e-4, ...
    'FunValCheck','off','OutputFcn',[],'PlotFcns',[]);
%% IGNORE THIS SECTION WHEN DEVELOPING CODE AND TESTING BY SECTION
% If just 'defaults' passed in, return the default options in X
if nargin==1 && nargout <= 1 && isequal(funfcn,'defaults')
    x = defaultopt;
    return
end

if nargin<3, options = []; end

% Detect problem structure input
if nargin == 1
    if isa(funfcn,'struct') 
        [funfcn,x,options] = separateOptimStruct(funfcn);
    else % Single input and non-structure
        error(message('MATLAB:fminsearch:InputArg'));
    end
end

if nargin == 0
    error(message('MATLAB:fminsearch:NotEnoughInputs'));
end
%%
% Check for non-double inputs
if ~isa(x,'double')
  error(message('MATLAB:fminsearch:NonDoubleInput'))
end

n = numel(x);
numberOfVariables = n;

printtype = optimget(options,'Display',defaultopt,'fast');
tolx = optimget(options,'TolX',defaultopt,'fast');
tolf = optimget(options,'TolFun',defaultopt,'fast');
maxfun = optimget(options,'MaxFunEvals',defaultopt,'fast');
maxiter = optimget(options,'MaxIter',defaultopt,'fast');
funValCheck = strcmp(optimget(options,'FunValCheck',defaultopt,'fast'),'on');

% In case the defaults were gathered from calling: optimset('fminsearch'):
if ischar(maxfun)
    if isequal(lower(maxfun),'200*numberofvariables')
        maxfun = 200*numberOfVariables;
    else
        error(message('MATLAB:fminsearch:OptMaxFunEvalsNotInteger'))
    end
end
if ischar(maxiter)
    if isequal(lower(maxiter),'200*numberofvariables')
        maxiter = 200*numberOfVariables;
    else
        error(message('MATLAB:fminsearch:OptMaxIterNotInteger'))
    end
end

switch printtype
    case {'notify','notify-detailed'}
        prnt = 1;
    case {'none','off'}
        prnt = 0;
    case {'iter','iter-detailed'}
        prnt = 3;
    case {'final','final-detailed'}
        prnt = 2;
    case 'simplex'
        prnt = 4;
    otherwise
        prnt = 1;
end
% Handle the output
outputfcn = optimget(options,'OutputFcn',defaultopt,'fast');
if isempty(outputfcn)
    haveoutputfcn = false;
else
    haveoutputfcn = true;
    xOutputfcn = x; % Last x passed to outputfcn; has the input x's shape
    % Parse OutputFcn which is needed to support cell array syntax for OutputFcn.
    outputfcn = createCellArrayOfFunctions(outputfcn,'OutputFcn');
end

% Handle the plot
plotfcns = optimget(options,'PlotFcns',defaultopt,'fast');
if isempty(plotfcns)
    haveplotfcn = false;
else
    haveplotfcn = true;
    xOutputfcn = x; % Last x passed to plotfcns; has the input x's shape
    % Parse PlotFcns which is needed to support cell array syntax for PlotFcns.
    plotfcns = createCellArrayOfFunctions(plotfcns,'PlotFcns');
end

header = ' Iteration   Func-count     min f(x)         Procedure';

%%
%----- YWS: Additional option to implement the parallel simplex -----
% Create parallel pool on cluster
tic
delete(gcp('nocreate'));
profileName = parallel.defaultClusterProfile();
clust = parcluster(profileName);
myMaxNumWorkers = clust.NumWorkers;
poolSize = min(myMaxNumWorkers, n); % the min() ensures there's at least one point left for the centroid
                                    % !!! use all available cores if possible - might be risky
                                    % I can make this myMaxNumWorkers - some_num
% If maxlab is specified, use the smaller of maxlab and calculated
% poolSize.
if ~isempty(maxlab)
    poolSize = min(poolSize,maxlab);
end
poolobj = parpool(clust, poolSize);
toc

%%
% Convert to function handle as needed.
funfcn = fcnchk(funfcn,length(varargin));

% Add a wrapper function to check for Inf/NaN/complex values
if funValCheck
    % Add a wrapper function, CHECKFUN, to check for NaN/complex values without
    % having to change the calls that look like this:
    % f = funfcn(x,varargin{:});
    % x is the first argument to CHECKFUN, then the user's function,
    % then the elements of varargin. To accomplish this we need to add the 
    % user's function to the beginning of varargin, and change funfcn to be
    % CHECKFUN.
    varargin = {funfcn, varargin{:}};
    funfcn = @checkfun;
end

n = numel(x);

% Initialize parameters
rho = 1; chi = 2; psi = 0.5; sigma = 0.5;
onesn = ones(1,n);
two2np1 = 2:n+1;

% YWS sets up the analogous paramters, given the pool size
% one2n = 1:n;
centroidIndex = 1:n+1-poolSize;
numCentPts = max(centroidIndex);
    
% Set up a simplex near the initial guess.
xin = x(:); % Force xin to be a column vector
v = zeros(n,n+1); fv = zeros(1,n+1);
v(:,1) = xin;    % Place input guess in the simplex! (credit L.Pfeffer at Stanford)
x(:) = xin;    % Change x to the form expected by funfcn
fv(:,1) = funfcn(x,varargin{:});
func_evals = 1;
itercount = 0;
how = '';
% Initial simplex setup continues later

% Initialize the output and plot functions.
if haveoutputfcn || haveplotfcn
    [xOutputfcn, optimValues, stop] = callOutputAndPlotFcns(outputfcn,plotfcns,v(:,1),xOutputfcn,'init',itercount, ...
        func_evals, how, fv(:,1),varargin{:});
    if stop
        [x,fval,exitflag,output] = cleanUpInterrupt(xOutputfcn,optimValues);
        if  prnt > 0
            disp(output.message)
        end
        return;
    end
end
%%
% Print out initial f(x) as 0th iteration
if prnt == 3
    disp(' ')
    disp(header)
    fprintf(' %5.0f        %5.0f     %12.6g         %s\n', itercount, func_evals, fv(1), how);
elseif prnt == 4
    clc
    formatsave.format = get(0,'format');
    formatsave.formatspacing = get(0,'formatspacing');
    % reset format when done
    oc1 = onCleanup(@()set(0,'format',formatsave.format));
    oc2 = onCleanup(@()set(0,'formatspacing',formatsave.formatspacing));
    format compact
    format short e
    disp(' ')
    disp(how)
    v
    fv
    func_evals
end
% OutputFcn and PlotFcns call
if haveoutputfcn || haveplotfcn
    [xOutputfcn, optimValues, stop] = callOutputAndPlotFcns(outputfcn,plotfcns,v(:,1),xOutputfcn,'iter',itercount, ...
        func_evals, how, fv(:,1),varargin{:});
    if stop  % Stop per user request.
        [x,fval,exitflag,output] = cleanUpInterrupt(xOutputfcn,optimValues);
        if  prnt > 0
            disp(output.message)
        end
        return;
    end
end

% Continue setting up the initial simplex.
% Following improvement suggested by L.Pfeffer at Stanford
usual_delta = 0.05;             % 5 percent deltas for non-zero terms
zero_term_delta = 0.00025;      % Even smaller delta for zero elements of x

%%----- YWS: I made this a parfor.
parfor j = 1:n
    y = xin;
    if y(j) ~= 0
        y(j) = (1 + usual_delta)*y(j);
    else
        y(j) = zero_term_delta;
    end
    v(:,j+1) = y;
    f = funfcn(y,varargin{:});
    fv(1,j+1) = f;
end

%%
% sort so v(1,:) has the lowest function value
[fv,j] = sort(fv);
v = v(:,j);
%%
how = 'initial simplex';
itercount = itercount + 1;
func_evals = n+1;
if prnt == 3
    fprintf(' %5.0f        %5.0f     %12.6g         %s\n', itercount, func_evals, fv(1), how)
elseif prnt == 4
    disp(' ')
    disp(how)
    v
    fv
    func_evals
end
% OutputFcn and PlotFcns call
if haveoutputfcn || haveplotfcn
    [xOutputfcn, optimValues, stop] = callOutputAndPlotFcns(outputfcn,plotfcns,v(:,1),xOutputfcn,'iter',itercount, ...
        func_evals, how, fv(:,1),varargin{:});
    if stop  % Stop per user request.
        [x,fval,exitflag,output] = cleanUpInterrupt(xOutputfcn,optimValues);
        if  prnt > 0
            disp(output.message)
        end
        return;
    end
end
exitflag = 1;
%%
% Main algorithm: iterate until 
% (a) the maximum coordinate difference between the current best point and the 
% other points in the simplex is less than or equal to TolX. Specifically,
% until max(||v2-v1||,||v3-v1||,...,||v(n+1)-v1||) <= TolX,
% where ||.|| is the infinity-norm, and v1 holds the 
% vertex with the current lowest value; AND
% (b) the corresponding difference in function values is less than or equal
% to TolFun. (Cannot use OR instead of AND.)
% The iteration stops if the maximum number of iterations or function evaluations 
% are exceeded

while func_evals < maxfun && itercount < maxiter
    if max(abs(fv(1)-fv(two2np1))) <= max(tolf,10*eps(fv(1))) && ...
            max(max(abs(v(:,two2np1)-v(:,onesn)))) <= max(tolx,10*eps(max(v(:,1))))
        break
    end
%%  
    % centroid
    % xbar = average of the numCentPts (n+1-poolSize) best points, 
    % since poolSize of the points will go through the simplex iteration
    xbar = sum(v(:,centroidIndex), 2)/numCentPts;
    currMinFv = fv(:,1);
    % We need:
    % - a cell array to store the procedure done to each point
    % - a vector to store if there each point undergoes a procedure
    % - a vector to store the number of func evals per point
    howCell = cell(1,poolSize);
    noAction = zeros(1,poolSize);
    funcEvalPerPoint = zeros(1,poolSize);
    
    fv_nextWorstPts = fv(numCentPts:n);
    func_evals_prePar = func_evals; % number of function evaluations from
                                    % the previous iteration
    
    % YWS: This is where the parallel simplex happens.
    % parWorker indexes the parallel worker (processor or core)
    parfor parWorker=1:poolSize
        % Initialize the point to evaluate
        xcurr = zeros(1,n);
        
        % For each of the points, identify the function value of the next
        % worst point
        nextWorstFv = fv_nextWorstPts(parWorker);
        
        % Compute the reflection point for each of the poolSize workers
        xr = (1 + rho)*xbar - rho*v(:,numCentPts + parWorker);
        xcurr(:) = xr; fxr = funfcn(xcurr,varargin{:});
        funcEvalPerPoint(parWorker) = funcEvalPerPoint(parWorker)+1;
%         xr = (1 + rho)*xbar - rho*v(:,end);
%         x(:) = xr; fxr = funfcn(x,varargin{:});
        func_evals = func_evals+1;
        
        if fxr < currMinFv
            % Calculate the expansion point
            xe = (1 + rho*chi)*xbar - rho*chi*v(:,numCentPts + parWorker);
            xcurr(:) = xe; fxe = funfcn(xcurr,varargin{:});
            funcEvalPerPoint(parWorker) = funcEvalPerPoint(parWorker)+1;
%             xe = (1 + rho*chi)*xbar - rho*chi*v(:,end);
%             x(:) = xe; fxe = funfcn(x,varargin{:});
            func_evals = func_evals+1;
            if fxe < fxr
                v(:,numCentPts + parWorker) = xe;
                fv(:,numCentPts + parWorker) = fxe;
                howCell(parWorker) = {['(lab ' num2str(parWorker) ') expand']};
            else
                v(:,numCentPts + parWorker) = xr;
                fv(:,numCentPts + parWorker) = fxr;
                howCell(parWorker) = {['(lab ' num2str(parWorker) ') reflect']};
            end
        else % fv(:,1) <= fxr
            if fxr < nextWorstFv
%             if fxr < fv(:,n)    
                v(:,numCentPts + parWorker) = xr;
                fv(:,numCentPts + parWorker) = fxr;
                howCell(parWorker) = {['(lab ' num2str(parWorker) ') reflect']};
%                 how = 'reflect';
            else % fxr >= fv(:,n)
                % Perform contraction
                if fxr < fv(:,numCentPts + parWorker)
                    % Perform an outside contraction
                    xc = (1 + psi*rho)*xbar - psi*rho*v(:,numCentPts + parWorker);
                    xcurr(:) = xc; fxc = funfcn(xcurr,varargin{:});
                    funcEvalPerPoint(parWorker) = funcEvalPerPoint(parWorker)+1;
                    func_evals = func_evals+1;

                    if fxc <= fxr
                        v(:,numCentPts + parWorker) = xc;
                        fv(:,numCentPts + parWorker) = fxc;
                        howCell(parWorker) = {['(lab ' num2str(parWorker) ') contract outside']};
                    else
                        % perform a shrink
                        howCell(parWorker) = {['(lab ' num2str(parWorker) ') shrink']};
                        noAction(parWorker) = 1;
                    end
                else
                    % Perform an inside contraction
                    xcc = (1-psi)*xbar + psi*v(:,numCentPts + parWorker);
                    xcurr(:) = xcc; fxcc = funfcn(xcurr,varargin{:});
                    funcEvalPerPoint(parWorker) = funcEvalPerPoint(parWorker)+1;
                    func_evals = func_evals+1;

                    if fxcc < fv(:,numCentPts + parWorker)
                        v(:,numCentPts + parWorker) = xcc;
                        fv(:,numCentPts + parWorker) = fxcc;
                        howCell(parWorker) = {['(lab ' num2str(parWorker) ') contract inside']};
                    else
                        % perform a shrink
                        howCell(parWorker) = {['(lab ' num2str(parWorker) ') shrink']};
                        noAction(parWorker) = 1;
                    end
                end
            end
        end
    end % parfor
                                
    % Only shrink the simplex if every evaluated point says so.
    if sum(noAction)==poolSize
        for j=two2np1
            v(:,j)=v(:,1)+sigma*(v(:,j) - v(:,1));
            x(:) = v(:,j); fv(:,j) = funfcn(x,varargin{:});
        end
        func_evals = func_evals + n;
    end
    
    [fv,j] = sort(fv);
    v = v(:,j);
    itercount = itercount + 1;
   
    % YWS: I turned this into a for loop (to print over the parWorkers)
    if prnt == 3
        for parWorker=1:poolSize
            if parWorker==1
                iterStr = sprintf('%5.0f',itercount);
            else
                iterStr = [repmat(' ',1,5-length(num2str(itercount))) repmat('-',1,length(num2str(itercount)))];
            end
            func_evals_prePar = func_evals_prePar + funcEvalPerPoint(parWorker);
            how = howCell{parWorker};
            fprintf(' %s        %5.0f     %12.6g         %s\n', iterStr, func_evals_prePar, fv(1), how)
        end
        % only uncomment the following if we really want to see what's going on
%         v
%         j
%         fv
        if func_evals_prePar ~= func_evals        
            fprintf(' %s        %5.0f     %12.6g         %s\n', iterStr, func_evals_prePar+n, fv(1), '[shrink simplex]')
        end
        
%         % Ad-hoc option added on 10/31/16: display the params with min function
%         % value every 5th iteration. The following condition should be
%         % commented out for most cases, unless the estimation takes a very
%         % long time and we want to see intermediate results.
%         if mod(itercount,5)==0
%             v(:,1)
%         end        
    elseif prnt == 4
        for parWorker=1:poolSize
            func_evals_prePar = func_evals_prePar + funcEvalPerPoint(parWorker);
            how = howCell{parWorker};
            disp(' ')
            disp(how)
            fprintf('func_evals: %d\n', func_evals_prePar)
        end
        v
        fv
    end   

    % OutputFcn and PlotFcns call
    if haveoutputfcn || haveplotfcn
        [xOutputfcn, optimValues, stop] = callOutputAndPlotFcns(outputfcn,plotfcns,v(:,1),xOutputfcn,'iter',itercount, ...
            func_evals, how, fv(:,1),varargin{:});
        if stop  % Stop per user request.
            [x,fval,exitflag,output] = cleanUpInterrupt(xOutputfcn,optimValues);
            if  prnt > 0
                disp(output.message)
            end
            return;
        end % if
    end % if
%%
end   % while

x(:) = v(:,1);
fval = fv(:,1);

output.iterations = itercount;
output.funcCount = func_evals;
output.algorithm = 'Nelder-Mead simplex direct search';

% OutputFcn and PlotFcns call
if haveoutputfcn || haveplotfcn
    callOutputAndPlotFcns(outputfcn,plotfcns,x,xOutputfcn,'done',itercount, func_evals, how, fval, varargin{:});
end

if func_evals >= maxfun
    msg = getString(message('MATLAB:fminsearch:ExitingMaxFunctionEvals', sprintf('%f',fval)));
    if prnt > 0
        disp(' ')
        disp(msg)
    end
    exitflag = 0;
elseif itercount >= maxiter
    msg = getString(message('MATLAB:fminsearch:ExitingMaxIterations', sprintf('%f',fval)));
    if prnt > 0
        disp(' ')
        disp(msg)
    end
    exitflag = 0;
else
    msg = ...
      getString(message('MATLAB:fminsearch:OptimizationTerminatedXSatisfiesCriteria', ...
               sprintf('%e',tolx), sprintf('%e',tolf)));
    if prnt > 1
        disp(' ')
        disp(msg)
    end
    exitflag = 1;
end

output.message = msg;

%%
%----- YWS: Additional steps to close out the parallel pool -----
delete(poolobj);

%--------------------------------------------------------------------------
function [xOutputfcn, optimValues, stop] = callOutputAndPlotFcns(outputfcn,plotfcns,x,xOutputfcn,state,iter,...
    numf,how,f,varargin)
% CALLOUTPUTANDPLOTFCNS assigns values to the struct OptimValues and then calls the
% outputfcn/plotfcns.
%
% state - can have the values 'init','iter', or 'done'.

% For the 'done' state we do not check the value of 'stop' because the
% optimization is already done.
optimValues.iteration = iter;
optimValues.funccount = numf;
optimValues.fval = f;
optimValues.procedure = how;

xOutputfcn(:) = x;  % Set x to have user expected size
stop = false;
% Call output functions
if ~isempty(outputfcn)
    switch state
        case {'iter','init'}
            stop = callAllOptimOutputFcns(outputfcn,xOutputfcn,optimValues,state,varargin{:}) || stop;
        case 'done'
            callAllOptimOutputFcns(outputfcn,xOutputfcn,optimValues,state,varargin{:});
        otherwise
            error(message('MATLAB:fminsearch:InvalidState'))
    end
end
% Call plot functions
if ~isempty(plotfcns)
    switch state
        case {'iter','init'}
            stop = callAllOptimPlotFcns(plotfcns,xOutputfcn,optimValues,state,varargin{:}) || stop;
        case 'done'
            callAllOptimPlotFcns(plotfcns,xOutputfcn,optimValues,state,varargin{:});
        otherwise
            error(message('MATLAB:fminsearch:InvalidState'))
    end
end

%--------------------------------------------------------------------------
function [x,FVAL,EXITFLAG,OUTPUT] = cleanUpInterrupt(xOutputfcn,optimValues)
% CLEANUPINTERRUPT updates or sets all the output arguments of FMINBND when the optimization
% is interrupted.

% Call plot function driver to finalize the plot function figure window. If
% no plot functions have been specified or the plot function figure no
% longer exists, this call just returns.
callAllOptimPlotFcns('cleanuponstopsignal');

x = xOutputfcn;
FVAL = optimValues.fval;
EXITFLAG = -1;
OUTPUT.iterations = optimValues.iteration;
OUTPUT.funcCount = optimValues.funccount;
OUTPUT.algorithm = 'Nelder-Mead simplex direct search';
OUTPUT.message = getString(message('MATLAB:fminsearch:OptimizationTerminatedPrematurelyByUser'));

%--------------------------------------------------------------------------
function f = checkfun(x,userfcn,varargin)
% CHECKFUN checks for complex or NaN results from userfcn.

f = userfcn(x,varargin{:});
% Note: we do not check for Inf as FMINSEARCH handles it naturally.
if isnan(f)
    error(message('MATLAB:fminsearch:checkfun:NaNFval', localChar( userfcn )));  
elseif ~isreal(f)
    error(message('MATLAB:fminsearch:checkfun:ComplexFval', localChar( userfcn )));  
end

%--------------------------------------------------------------------------
function strfcn = localChar(fcn)
% Convert the fcn to a string for printing

if ischar(fcn)
    strfcn = fcn;
elseif isa(fcn,'inline')
    strfcn = char(fcn);
elseif isa(fcn,'function_handle')
    strfcn = func2str(fcn);
else
    try
        strfcn = char(fcn);
    catch
        strfcn = getString(message('MATLAB:fminsearch:NameNotPrintable'));
    end
end