%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%LTCH moment sensitivity to perturbation of parameter
%we produce 2 sets of outputs
% 1: average sensitivity of 4 groups of discharge moments to 3 groups of
% parameter (initial health, health, preference)

% 2: detailed sensitivity of all discharge moments to intial health mean
% (mu0) and logit error (scale)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [] = params_perturb(paramsStruct, retStruct)

    %extract dimensions and estimation results
    TT=paramsStruct.TT;
    backInductT=paramsStruct.backInductT;
    days=(1:TT) -backInductT; 
    
    params_all_hat = retStruct.params_all_hat;
    gmmSE   = retStruct.gmmSE;
    moments = paramsStruct.moments;
    W_0     = paramsStruct.W_0;    
    
    %% Perturbation
    %perturb 1 sd of each parameter
    %create another version of scale parameter preturbation where we multiple
    %it by 5
    %collect moment outputs: each parameter, estimates vs perturb, TT*3 discharge shares
    sd =[0,1];
    sdx=[1,5];
    
    %post-PPS health and preference parameters
    p0 = find(strcmp(paramsStruct.paramsEstimated,'muPost'));
    p1 = find(strcmp(paramsStruct.paramsEstimated,'mu0Post'));
    p2 = find(strcmp(paramsStruct.paramsEstimated,'scale'));
    
    postPPSparam = p0:p2;
    out_shares=zeros(length(postPPSparam), length(sd), TT, 3);
    out_shares2=out_shares;

    %we don't want to adjust health grid when we perturb health parameter
    % so we introduce another field in paramsStruct
    paramsStruct.fixHGrid=params_all_hat(6:8);

    for i=1:length(postPPSparam)
        dpar = postPPSparam(i);

        for ii=1:length(sd)
            %perturbed parameters
            new_params=params_all_hat;
            new_params(dpar)= new_params(dpar) + sd(ii).*gmmSE(dpar);

            %moments
            %fit parameter into standard objective function
            [~, par_hat] = objective_function(new_params, moments, W_0, paramsStruct);

            %get post-PPS discharge moments TT*3
            discharge_shares = reshape(par_hat.moments_hat_jacobian(TT*3+1:TT*6), TT, 3);

            %vectorize all discharge shares 
            out_shares(i,ii,:,:) =discharge_shares; 
            out_shares2(i,ii,:,:)=discharge_shares; 


            %for scale parameter different perturbation
            if i==length(postPPSparam)
                new_params=params_all_hat;
                new_params(dpar)= new_params(dpar).*sdx(ii);

                [~, par_hat] = objective_function(new_params, moments, W_0, paramsStruct);

                %get post-PPS discharge moments TT*3
                out_shares2(i,ii,:,:) = reshape(par_hat.moments_hat_jacobian(TT*3+1:TT*6), TT, 3); 
            end            
        end
    end

    %work out the absolute difference
    out_shares  = squeeze(abs(out_shares(:,1,:,:)  - out_shares(:,2,:,:)));
    out_shares2 = squeeze(abs(out_shares2(:,1,:,:) - out_shares2(:,2,:,:)));


    %% group change in discharge moments

    % we group discharge shares to days -15 to -11, -10 to -3, -2 to 10, 11 to 45
    Tgrp = zeros(TT,4);
    Tgrp(1:5,   1)=1;
    Tgrp(6:13,  2)=1;
    Tgrp(14:26, 3)=1;
    Tgrp(27:end,4)=1;

    %we group parameter to initial health, health, preference
    Pgrp = zeros(10,3);
    Pgrp(4:5,  1)=1;
    Pgrp(1:3,  2)=1;
    Pgrp(6:end,3)=1;


    %normalize by the number of days * number of parameters
    outScale=10000./((sum(Tgrp)'*ones(1,3)).*(ones(4,1)*sum(Pgrp)));

    %stack each of three discharge destination
    perturbOut=[];
    for i=1:3
        perturbOut = [perturbOut; ...
                    (Pgrp'*squeeze(out_shares(:,:,i))*Tgrp)'.*outScale];
    end


    %% select parameter to show detailed results
    %focus on mu0 and scale 
    perturbDetail = [];
    for i=[p1,p2]
        perturbDetail=[perturbDetail, squeeze(out_shares2(postPPSparam==i, :, :))];
    end

    %% output perturbOut and perturbDetail to excel
    xlsOutFile=paramsStruct.xlsOutFile;
    xlsSheetNameA = ['perturb4x3', paramsStruct.xlsShtNm];
    xlsSheetNameB = ['perturbPar', paramsStruct.xlsShtNm];
    
    %write 4x3 matrix of grouped perturbation results
    xlswrite(xlsOutFile,perturbOut,xlsSheetNameA, 'C2')

    %write titles
    colTit = {'Initial health','Health process','Preference'};
    rowTit = {'Early';'Pre-threshold';'Near-threshold';'Post-threshold'};
    rowTit = [{'Upstream';'';'';'';'Downstream';'';'';'';'Death';'';'';''}, ...
              [rowTit;rowTit;rowTit]];
    xlswrite(xlsOutFile,colTit,xlsSheetNameA,'C1')
    xlswrite(xlsOutFile,rowTit,xlsSheetNameA,'A2')


    %write detailed perturbation for selected parameter
    xlswrite(xlsOutFile,[days', perturbDetail],xlsSheetNameB,'A3')
    
    colTit = {'Upstream','Downstream','Death'};
    colTit = [{'Days relative', '','mu0','','','scale',''}; ['to SSO', colTit, colTit]];
    xlswrite(xlsOutFile,colTit,xlsSheetNameB,'A1')

    %print time stamp
    print_timestamp(xlsOutFile, xlsSheetNameA, paramsStruct.version, paramsStruct.run);
    print_timestamp(xlsOutFile, xlsSheetNameB, paramsStruct.version, paramsStruct.run);
end
