%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%LTCH continuation values and their sensitivity to perturbation of parameter
%we produce 3 sets of outputs
% 1: plot continuation values (Vs) at different time and health status to
% gain intuition of the dynamic nature of the model

% 2: plot probability of staying at LTCH until SSO
% across health status. This illustrates the change in trade off prior to 
% SSO

% 3: sensitivity of Vs to perturbation of selected parameters: 
% gamma, scale, and preference
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




function [] = plot_V(paramsStruct, retStruct)

    %parameter estimates and moments
    params_all_hat = retStruct.params_all_hat;
    z_post  = retStruct.z_post;
    
    %timing
    TT=paramsStruct.TT;
    backInductT=paramsStruct.backInductT;
    n = paramsStruct.n;
    days=(1:TT) -backInductT; 
    
    %% get continuation Vs
    %slightly different numeric setting
    paramsStruct.valFuncFloor_pre  = -2e3;
    paramsStruct.valFuncTrim_back0 = 1;
    paramsStruct.plotV=1;
    
    %indicate that we don't want to change the health grid in perturbation
    %exercise
    health_par = find(strcmp(paramsStruct.paramsEstimated,'muPost')):find(strcmp(paramsStruct.paramsEstimated,'sigmaPost'));
    paramsStruct.fixHGrid=params_all_hat(health_par); 
 
    [V_A_tn,V_S_tn,V_l_tn] = organize_V(params_all_hat, paramsStruct);

    %%  change in Vs with perturbed parameter
    up_h  =20;      %health status 0.0188 based on z_post(up_h-1)
    down_h=200;     %health status 10.5135 based on z_post(down_h-1)

    % first two: 
    % increase gamma or sigma by X (parameter 11 and 15)
    % last three:
    % increase parameters v1a v0s v1s such that 
    % v_u(h) and v_d(h) increase by X% (parameter 12 - 14)
    perturb=[1,1.25;   ...
             1,1.5;    ...
             1,0.2;    ...  %80%  for v1a
             1,0.6467; ...  %400% for v0s
             1,1.3876];     %400% for v1s
    parN = {'gamma', 'scale', 'v1A', 'v0S', 'v1S'};
    parChoose=[];
    for par=parN
        parChoose=[parChoose, find(strcmp(paramsStruct.paramsEstimated,par))];
    end    
    storeV=zeros(size(perturb,1),size(perturb,2),2,2);
    [n_par, n_per]=size(perturb);
    meanV=zeros(TT,2,n_per,n_par);

    for j=1:n_par
        %perturb which parameter
        var=parChoose(j);

        for i=1:n_per
            params_perturb =params_all_hat;
            params_perturb(var)=params_perturb(var)*perturb(j,i);

            [V_A_perturb,V_S_perturb,V_l_perturb] = organize_V(params_perturb, paramsStruct);
            
            %record Vs
            storeV(j,i,1,:)=[V_A_perturb(1,up_h), V_S_perturb(1,down_h)];
            storeV(j,i,2,:)=V_l_perturb(backInductT,[up_h,down_h]); 

            %record utility
            meanV(:,1,i,j)=V_A_perturb(:,up_h)  -V_l_perturb(:,up_h);
            meanV(:,2,i,j)=V_S_perturb(:,down_h)-V_l_perturb(:,down_h);
        end
    end


    %% probability of making to SSO 
    
    %health transition
    P_D=retStruct.P_D_post.P_D_16;
    
    %post SSO stay prob
    Q_l_post = retStruct.Q_l_post;
    
    % we also solve the P(getting SSO) with backward induction
    P_sso=ones(backInductT, n); 
    for t = backInductT-1:-1:1
        P_sso(t,:)= P_D * (Q_l_post(t,:)' .* P_sso(t+1,:)');
    end

  

    %% write to excel
    xlsOutFile=paramsStruct.xlsOutFile;
    
    % 1: write Vs
    xlsSheet_V = ['plot_V', paramsStruct.xlsShtNm];
    
    outV = [(1:length(z_post))+1; z_post; ...
        V_A_tn(1,:); V_S_tn(1,:); V_l_tn(1:16,:)]';
    xlswrite(xlsOutFile, outV, xlsSheet_V, 'A3')
    
    %write titles
    colTit = {'Health grid','Health status','V: Payoff upstream','V: Payoff downstream', 'V: Payoff within LTCH: profit + continuation on each day (relative to SSO)'};
    vt = -15:0;
    xlswrite(xlsOutFile,colTit,xlsSheet_V,'A1')
    xlswrite(xlsOutFile,vt,    xlsSheet_V,'E2')
   
    
    % 2: write Pr[SSO]
    xlsSheet_sso = ['PrSSO', paramsStruct.xlsShtNm];
    
    outV = [(1:length(z_post))+1; z_post; P_sso(:,2:end)]';
    xlswrite(xlsOutFile,outV,xlsSheet_sso, 'A3')

    %write titles
    colTit = {'Health grid','Health status','Probability of making to SSO day, conditional on each health status on each day (relative to SSO)'};
    xlswrite(xlsOutFile,colTit,xlsSheet_sso,'A1')
    xlswrite(xlsOutFile,vt,xlsSheet_sso,'C2')


    % 3:V perturbation
    xlsSheet_a = ['perturbVa_', paramsStruct.xlsShtNm];
    xlsSheet_s = ['perturbVd_', paramsStruct.xlsShtNm];
    
    %parameter and health values
    parV = repmat(params_all_hat(parChoose),n_per,1).*perturb';
    h_a = repmat([up_h;z_post(up_h)],1,n_par*n_per);
    h_d = repmat([down_h;z_post(down_h)],1,n_par*n_per);
    V_a = [reshape(storeV(:,:,1,1)',1,[]); reshape(storeV(:,:,2,1)',1,[])];
    V_d = [reshape(storeV(:,:,1,2)',1,[]); reshape(storeV(:,:,2,2)',1,[])];
    xlswrite(xlsOutFile,[parV(:)'; h_a; V_a],xlsSheet_a,'B1')
    xlswrite(xlsOutFile,[parV(:)'; h_d; V_d],xlsSheet_s,'B1')

    %row title
    rowTit = {'Parameter Values'; 'health grid'; 'health status'; 'V discharge'; 'V at LTCH at SSO'};
    xlswrite(xlsOutFile,rowTit,xlsSheet_a,'A1')
    xlswrite(xlsOutFile,rowTit,xlsSheet_s,'A1')

    %column title
    col1 = ['Days relative', reshape(repmat(parN,2,1),1,[])];
    col2 = ['to SSO', repmat({'base', 'perturbed'}, 1, n_par)];
    colTit = [col1; col2];
    xlswrite(xlsOutFile,colTit,xlsSheet_a,'A6')
    xlswrite(xlsOutFile,colTit,xlsSheet_s,'A6')

    %upstream vs downstream
    outVa = [days', reshape(meanV(:,1,:,:), TT,[])];
    outVd = [days', reshape(meanV(:,2,:,:), TT,[])];
    xlswrite(xlsOutFile,outVa,xlsSheet_a,'A8')
    xlswrite(xlsOutFile,outVd,xlsSheet_s,'A8')


    %print time stamp
    print_timestamp(xlsOutFile, xlsSheet_V,   paramsStruct.version, paramsStruct.run, 'A300');
    print_timestamp(xlsOutFile, xlsSheet_sso, paramsStruct.version, paramsStruct.run, 'A300');
    print_timestamp(xlsOutFile, xlsSheet_a,   paramsStruct.version, paramsStruct.run);
    print_timestamp(xlsOutFile, xlsSheet_s,   paramsStruct.version, paramsStruct.run);
end 

%% organize Vs
function [V_A_tn,V_S_tn,V_l_tn] = organize_V(params_all, paramsStruct)
      
    %timing
    TT=paramsStruct.TT;
    backInductT=paramsStruct.backInductT;
    
    %simulate forward using the objective function 
    % we impose slightly different cut off for large values of V in logit
    % formula to get a more accurate measure of continuation value (double
    % check if this is necessary)
    [~, ~, ~, ~, ...
        ~, ~, ~, ...
        ~, ~, V_A_post, V_S_post, V_l_post, ...
        ~, ~, l_preSSO_post, ...
        ~, ~, ~, ~, ~, ~, ~, ~, ~] = ...
            evaluate_objective('post', params_all, paramsStruct);
            
    V_A_tn = repmat(V_A_post(2:end),TT,1);
    V_S_tn = repmat(V_S_post(2:end),TT,1);
    V_l_tn = [l_preSSO_post(:,2:end); repmat(V_l_post(2:end), TT-backInductT+1,1)];  
end
