%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make plots for a chosen version and run, as defined in main
% This assumes that estimates have already been loaded. If not,
% simply type "load_estimates" after declaring globals.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COUNTERFACTUALS (cf)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  [paramsStruct, retStruct] = plot_model_results(paramsStruct, retStruct)
    fprintf('-- Make plots. \n\n');
    close all;

    %% 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % plots model fit for all destinations and perios
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    close all;
    if ~strcmpi(paramsStruct.momentsUsed, 'post')
        plot_model_fit('a', 'pre', paramsStruct, retStruct);
        plot_model_fit('s', 'pre', paramsStruct, retStruct);
        plot_model_fit('m', 'pre', paramsStruct, retStruct);
    end
    close all;
    if ~strcmpi(paramsStruct.momentsUsed, 'pre')
        plot_model_fit('a', 'post', paramsStruct, retStruct);
        plot_model_fit('s', 'post', paramsStruct, retStruct);
        plot_model_fit('m', 'post', paramsStruct, retStruct);
    end
    
    
    %% 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % OUTPUT MOMENT FITS TO XLS
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    export_moment_fit_xls(paramsStruct, retStruct);

    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % POLICY FUNCTIONS
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if strcmp(paramsStruct.grandver, 'v18')
        if strcmpi(paramsStruct.momentsUsed, 'pre')
            temp_periods = {'pre'};
        elseif strcmpi(paramsStruct.momentsUsed, 'post')
            temp_periods = {'post'};
        else
            temp_periods = {'pre','post'};
        end
        close all;
        flag_outputPoliFuncPlot = true; % Whether to create PDF output.
        for period = temp_periods
            retStruct = plot_policy_function(cell2mat(period), '', ...
                paramsStruct, retStruct, flag_outputPoliFuncPlot);
        end
    end
end


%% plot discharge shares moment fits
function [] = plot_model_fit(destID, period, paramsStruct, retStruct)

    % plot config
    [days, ~, ~, ~, myXMin, myXMax, ~] = plot_setup(paramsStruct);

    fldTag = [destID, '_', period];

    % select the targeted (actual) and estimated moments based on the discharge destination
    share_actual    = paramsStruct.(['share_', fldTag]);
    share_estimated = retStruct.(['share_', fldTag, '_hat']);
    maxDay = min(numel(share_actual),numel(share_estimated));
    destShort  = set_plot_names(destID);


    fig_handle = figure; % this creates a figure handle (and a new figure object)
    b = bar(days, [share_actual(1:maxDay) share_estimated(1:maxDay)]);
    set(b(1),'FaceColor',[0 0 1]);
    set(b(2),'FaceColor',[.9 .9 .9]);
    xlim([myXMin myXMax]);
    xlabel('Days','FontSize',18)
    ylabel(['Share Discharged to ' destShort],'FontSize',18);
    title({['Model Fit Plot for the ' period '-period'], ['Share Discharged to ' destShort], ''},'FontSize',18);
    leg = legend('Data','Simulated'); set(leg, 'FontSize', 18); legend boxoff;
    out_file = fullfile(paramsStruct.outDir, 'Figures', ['model_fit_' destShort '_' period '_' paramsStruct.version '.pdf']);
    print('-painters','-dpdf','-r300',out_file);
    fprintf('Printed %s\n', out_file);
end


%% export moment fits to XLS
function [] = export_moment_fit_xls(paramsStruct, retStruct)

    TT = paramsStruct.TT;
    
    
    if strcmpi(paramsStruct.momentsUsed, 'pre')
        temp_periods = {'pre'};
    elseif strcmpi(paramsStruct.momentsUsed, 'post')
        temp_periods = {'post'};
    else
        temp_periods = {'pre','post'};
    end

    %for each period
    for periodID = temp_periods
        periodID = cell2mat(periodID);

        % Reset output headers and columns for each period.
        temp_colnames = {'days'};
        temp_outmat = paramsStruct.days(:);
        
        % Choose moments based on objType.
        switch(paramsStruct.objType)
            case('discShares')
                momentsTypes = {'share_'};
            case('discShares_nonltchPay')
                if strcmpi(periodID, 'pre') && paramsStruct.ind_omitPrePPSNonLtch
                    momentsTypes = {'share_'};
                else
                    momentsTypes = {'share_', 'nonltchPay_'};
                end
            otherwise
                fprintf('Invalid paramsStruct.objType: %s', paramsStruct.objType);
        end

        for momentsType = momentsTypes
            momentsType = cell2mat(momentsType);

            switch(momentsType)
                case('share_')
                    destIDs = {'a', 's', 'm'};
                case('nonltchPay_')
                    destIDs = {'a', 's'};
                otherwise
                    fprintf('Invalid momentsType: %s', momentsType);
            end

            for destID = destIDs
                destID = cell2mat(destID);

                for suffix = {'', '_hat'}
                    suffix = cell2mat(suffix);
                    if isempty(suffix)
                        strct = 'paramsStruct.';
                    else
                        strct = 'retStruct.';
                    end
                    temp_var = [momentsType destID '_' periodID suffix];
                    % The target post-LTCH moments have a prefix when we use
                    % the aggregated version. Also, the aggregated moments are
                    % only stored in paramsStruct.
                    if isempty(suffix) && strcmp(momentsType, 'nonltchPay_') 
                        temp_var = ['agg_' temp_var];
                    end

                    temp_colnames = [temp_colnames temp_var];
                    temp_numPads = TT - eval(['numel(' [strct temp_var] ')']);
                    temp_padded = eval(['[' [strct temp_var] '; NaN * ones(temp_numPads,1)]']);
                    temp_outmat = [temp_outmat temp_padded];
                end
            end
        end

        %file and sheet names
        xlsSheetName = ['moments_' periodID '_' paramsStruct.xlsShtNm];
        xlsOutFile = paramsStruct.xlsOutFile;

        %write outputs
        xlswrite(xlsOutFile,temp_colnames,xlsSheetName,'A1');
        xlswrite(xlsOutFile,temp_outmat,xlsSheetName,'A2');

        %print time stamp
        print_timestamp(xlsOutFile, xlsSheetName, paramsStruct.version, paramsStruct.run);
    end
end




