%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Call this script to set global cell arrays.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize structs as containers for user-input parameters and
% model-returned objects.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
paramsStruct = struct();    % To contain parameter names and values.

temp_verArray = strsplit(version,'_');
estType = cell2mat(temp_verArray(2));
if numel(temp_verArray)==3
    momentsUsed = 'pre_post';
    paramsStruct.estTypeSuffix = '';
else
    switch cell2mat(temp_verArray(3))
        case('c')
            momentsUsed = 'pre';
            paramsStruct.estTypeSuffix = 'c';
        case('d')
            momentsUsed = 'post';
            paramsStruct.estTypeSuffix = 'd';
        otherwise
            fprintf('Invalid momentsUsed: %s.\n', momentsUsed);
            return
    end
end
clear temp_verArray;

switch(grandver)
    case('v17')
        objType = 'discShares';
    case('v18')
        objType = 'discShares_nonltchPay';
    otherwise
        fprintf('Invalid grandver: %s.\n', grandver);
        return
end

% This indicator, when specified to be "true", omits the pre-PPS post-LTCH
% moments (we might not have enough identifying variation in health to pin
% down the post-LTCH payments vs. health of dischargees model).
ind_omitPrePPSNonLtch = true;


paramsStruct.estType = estType;
paramsStruct.momentsUsed = momentsUsed;
paramsStruct.objType = objType;
paramsStruct.nonLtchPayType = 'Aggregated by pre-/post-SSO';
paramsStruct.ind_omitPrePPSNonLtch = ind_omitPrePPSNonLtch; 

% model type
hosp = '_pool';
paramsStruct.hosp = hosp;

% Post-LTCH model
paramsStruct.nonltchPayModel = 'nlgmm'; 
nonltchPayModel = paramsStruct.nonltchPayModel;
retStruct = paramsStruct;   % To contain objects returned in estimation.


%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OUTPUT TAGS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% name tag for output
xlsShtNm = [paramsStruct.estType,paramsStruct.estTypeSuffix, paramsStruct.hosp];
if strcmpi(paramsStruct.objType,'discShares_nonltchPay')
    xlsShtNm = ['Z', xlsShtNm];
end
paramsStruct.xlsShtNm=xlsShtNm;

% output XLS
paramsStruct.xlsOutFile=fullfile(projroot, 'Output', 'Estimates_Log.xlsx');

% output directory for figurse
paramsStruct.outDir = fullfile(projroot, 'Output', filesep);   

% model version
paramsStruct.grandver = grandver;
paramsStruct.version = version;
paramsStruct.run = run;

% store workspace 
paramsStruct.outMat = ['Estimates_' version '_run_' run];


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MODEL CONFIGURATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n = 250;    % Number of health grid points.
T = 16;     % The SSO threshold day, when the payment jumps in the post-period.
TT = 61;    % Total number of days in our model.
paramsStruct.n = n;
paramsStruct.T = T; % SSO day
paramsStruct.TT = TT;

days = -15:1:(TT-T);
paramsStruct.days = days;

% We need a backInductT (backward induction T) to
% specify the day to start doing backward induction. This is because we
% have a complication when we allow the health process to be t-varying
% beyong t=T.

params_grid = [n, T, TT];
paramsStruct.backInductT = T;
paramsStruct.params_grid = params_grid;

% Penalties to enforce in the objective function.
paramsStruct.penalty_rho = 1e9;
paramsStruct.penalty_sigma = 1e9;

%--- Initial values for non-linear post-LTCH payments model: [zeta0; zeta1] ---
% Note: these initial values are only used when objType = 'discShares',
% i.e. we estimate the post-LTCH payments model in a separate model. Otherwise,
% if objType = 'discShares_nonltchPay', the initial values for zetas should
% be defined in params_all_0_pool
% above.
if strcmpi(paramsStruct.objType,'discShares')
    paramsStruct.zeta_a_pre_0 = [5.58959578089000e-41,-15.2065100541603];
    paramsStruct.zeta_s_pre_0 = [0.025944820320712, 0.491056770647924];
    paramsStruct.zeta_a_post_0 = [7.45391739331514,-1.56495804075796];   
    paramsStruct.zeta_s_post_0 = [11.2358312867825,-0.0645685423215771];
end

%--- Initial values for non-linear post-LTCH days model: [coefDays0; coefDays1] ---
% Note: we always estimate the post-LTCH DAYS model outside of the main
% model, so these need to be sepcified irrespective of grandver/objType.
paramsStruct.coefDays_a_pre_0 = [1,-5];
paramsStruct.coefDays_s_pre_0 = [0.0000164858999506098,0.517385791162492];
paramsStruct.coefDays_a_post_0 = [64.2580022972884, 0.745709771965486];
paramsStruct.coefDays_s_post_0 = [121.151186815632, -0.0785037212277011];

% Payments Schedule: pre-PPS period (Jan 2000 - Sep 2002); input to objective function.
p_perdiem_pre      =  1071.4677;

% Payments Schedule: (post-)PPS period (Oct 2007 - Sep 2012); input to objective function.
p_preSSO_perdiem_post  =  1379.6018 ;
p_preSSO_T_post        =  13624.6128;


%LTCH reported cost
ltchRepCostPost = 1426;
paramsStruct.repCostPerDay_pre  = 1068/ltchRepCostPost;   %ratio of pre to post cost
paramsStruct.repCostPerDay_post = 1;           %we estimate post-PPS cost


% Medicare payments per Day (2012 $)
% Numbers taken from the LTCH column.
% Used in create_counterfactual_payment_schedules.
% For example, one scenario we consider is pay LTCHs their "opportunity"
% costs of SNF payments.
%ratio of ACH/SNF cost to LTCH cost in post-PPS
paramsStruct.aPayPerDay_pre  = 1671;    % a = ACH
paramsStruct.aPayPerDay_post = 2079;    % a = ACH
paramsStruct.sPayPerDay_pre  = 414;     % s = SNF
paramsStruct.sPayPerDay_post = 507;     % s = SNF

target_moments_input  = 'target_pool';
superset_params_all_0 = params_all_0_pool;

paramsStruct.p_perdiem_pre  = p_perdiem_pre;
paramsStruct.p_perdiem_post = p_preSSO_perdiem_post;

paramsStruct.p_postSSO_pre  = p_perdiem_pre;
paramsStruct.p_preSSO_pre   = p_perdiem_pre*ones(T,1);
paramsStruct.p_postSSO_post = 0;
p_preSSO_post    = p_preSSO_perdiem_post*ones(T,1);
p_preSSO_post(T) = p_preSSO_T_post;
paramsStruct.p_preSSO_post  = p_preSSO_post;

%%
% Import target/actual moments.
in_file = fullfile(dataroot, target_moments_input);
load(in_file);
fprintf('%s -- Imported target moments (discharges) from %s. \n\n',char(datetime('now')),target_moments_input);

% Discharge shares as moments.
% These are for Days -15 to 45.
paramsStruct.share_a_pre  = share_a_pre;
paramsStruct.share_s_pre  = share_s_pre;
paramsStruct.share_m_pre  = share_m_pre;
paramsStruct.share_a_post = share_a_post;
paramsStruct.share_s_post = share_s_post;
paramsStruct.share_m_post = share_m_post;

% Import post-LTCH payments.
% In our GMM non-linear payments model, we use levels payments and days as 
% the outcomes.
% We now also consider whether we want to aggregate post-LTCH payments by
% pre-/post-SSO, and if so, we need to construct an aggregation matrix.
for temp_period = {'pre','post'}
    temp_period = cell2mat(temp_period);
    temp_matfile = matfile(fullfile(dataroot, ['nonLtchPay_' temp_period hosp]));

    for temp_dest = {'a','s'}
        temp_dest = cell2mat(temp_dest);

        % Create aggregation matrix if we want to aggregate post-LTCH
        % payments to just pre- and post-SSO moments.
        % These matrices differ by pre-/post-PPS and upstream/downstream
        temp_aggMat = 1;
        temp_share = eval(['share_' temp_dest '_' temp_period '(:)''']);
        temp_aggMat = [temp_share; temp_share];

        % First row corresponds to pre-SSO (<T), second row post-SSO
        % (>=T). In each row, the columns not corresponding to the same
        % relative SSO period should be zeros.
        temp_aggMat(1,T:end) = zeros(1,TT-T+1);
        temp_aggMat(2,1:T-1) = zeros(1,T-1);

        % Rescale so each row sums up to 1.
        temp_aggMat = temp_aggMat ./ (sum(temp_aggMat,2)*ones(1,TT));

        % Identify which non-aggregated post-LTCH quantities to extract.
        temp_nonltchPay  = temp_matfile.(['price_' temp_dest '_ub']);
        temp_nonltchDays = temp_matfile.(['los_' temp_dest '_ub']);
        
        % Save the day-by-day post-LTCH quantities.
        % Post-LTCH payments.
        flgTag=[temp_dest '_' temp_period];
        paramsStruct.(['nonltchPay_' flgTag])  = temp_nonltchPay;
        % Post-LTCH days.
        paramsStruct.(['nonltchDays_' flgTag]) = temp_nonltchDays;

        % Pre-multiply post-LTCH quantities by the aggregation matrix, 
        % then save to paramsStruct.
        paramsStruct.(['nonLtchPayAggMat_' flgTag]) = temp_aggMat;
        paramsStruct.(['agg_nonltchPay_' flgTag])   = temp_aggMat * temp_nonltchPay;
        paramsStruct.(['agg_nonltchDays_' flgTag])  = temp_aggMat * temp_nonltchDays;
    end
end

% Assemble the discharges as moments vectors.
moments_pre  = [paramsStruct.share_a_pre;  paramsStruct.share_s_pre;  paramsStruct.share_m_pre];
moments_post = [paramsStruct.share_a_post; paramsStruct.share_s_post; paramsStruct.share_m_post];
moments      = [moments_pre; moments_post];

% Post-LTCH payments as moments: only in v18, not v17.
% Append post-LTCH payments to the pre-existing moments vectors.
if strcmpi(paramsStruct.objType, 'discShares_nonltchPay')
    moments_post    =   [moments_post; ...
                        paramsStruct.agg_nonltchPay_a_post; paramsStruct.agg_nonltchPay_s_post];
    % We have a toggle to decide with we want to omit the post-LTCH
    % payments moments in pre-PPS.
    if paramsStruct.ind_omitPrePPSNonLtch
        moments     =   [moments; ...
                        paramsStruct.agg_nonltchPay_a_post; paramsStruct.agg_nonltchPay_s_post];
    else
        moments_pre =   [moments_pre; ...
                        paramsStruct.agg_nonltchPay_a_pre;  paramsStruct.agg_nonltchPay_s_pre];
        moments     =   [moments; ...
                        paramsStruct.agg_nonltchPay_a_pre;  paramsStruct.agg_nonltchPay_s_pre; ...
                        paramsStruct.agg_nonltchPay_a_post; paramsStruct.agg_nonltchPay_s_post];
    end
end
paramsStruct.moments        = moments;
paramsStruct.moments_pre    = moments_pre;
paramsStruct.moments_post   = moments_post;

%%
% Create the non-zero diagonal for the weighting matrix.
w_0_a_pre = zeros(TT,1); w_0_s_pre = zeros(TT,1); w_0_m_pre = zeros(TT,1);
w_0_a_post = zeros(TT,1); w_0_s_post = zeros(TT,1); w_0_m_post = zeros(TT,1);

for i = 1:TT
    w_0_s_pre(i)    = (1-abs((i-T)/TT));
    w_0_a_pre(i)    = w_0_s_pre(i)*(mean(paramsStruct.share_s_pre)/mean(paramsStruct.share_a_pre));
    w_0_m_pre(i)    = w_0_s_pre(i)*(mean(paramsStruct.share_s_pre)/mean(paramsStruct.share_m_pre));
    
    w_0_s_post(i)   = (1-abs((i-T)/TT));
    w_0_a_post(i)   = w_0_s_post(i)*(mean(paramsStruct.share_s_post)/mean(paramsStruct.share_a_post));
    w_0_m_post(i)   = w_0_s_post(i)*(mean(paramsStruct.share_s_post)/mean(paramsStruct.share_m_post));
end

% weight multiplier for ACH
weight_a_multiplier = 1;
paramsStruct.weight_a_multiplier = weight_a_multiplier;

% Create the weighting matrix with a non-zero diagonal, and zero everywhere else.
% Note that we have a weight_a_multiplier, which can be adjusted to place
% more or less weight on matching the discharge to ACH patterns.
temp_diag_pre   = [weight_a_multiplier*w_0_a_pre;  w_0_s_pre;  w_0_m_pre];
temp_diag_post  = [weight_a_multiplier*w_0_a_post; w_0_s_post; w_0_m_post];
temp_diag       = [temp_diag_pre; temp_diag_post];

% Modify the weighting matrix for v18: use the post-LTCH payments as moments.
if strcmpi(paramsStruct.objType, 'discShares_nonltchPay')
    w_0_nonltchPay_a_pre = zeros(TT,1); w_0_nonltchPay_s_pre = zeros(TT,1);
    w_0_nonltchPay_a_post = zeros(TT,1); w_0_nonltchPay_s_post = zeros(TT,1);

    for i = 1:TT
        w_0_nonltchPay_a_pre(i)  = w_0_s_pre(i)*(mean(paramsStruct.share_s_pre)/mean(paramsStruct.nonltchPay_a_pre));
        w_0_nonltchPay_s_pre(i)  = w_0_s_pre(i)*(mean(paramsStruct.share_s_pre)/mean(paramsStruct.nonltchPay_s_pre));
        w_0_nonltchPay_a_post(i) = w_0_s_post(i)*(mean(paramsStruct.share_s_post)/mean(paramsStruct.nonltchPay_a_post));
        w_0_nonltchPay_s_post(i) = w_0_s_post(i)*(mean(paramsStruct.share_s_post)/mean(paramsStruct.nonltchPay_s_post));
    end

    w_0_nonltchPay_a_pre  = paramsStruct.nonLtchPayAggMat_a_pre  * w_0_nonltchPay_a_pre;
    w_0_nonltchPay_s_pre  = paramsStruct.nonLtchPayAggMat_s_pre  * w_0_nonltchPay_s_pre;
    w_0_nonltchPay_a_post = paramsStruct.nonLtchPayAggMat_a_post * w_0_nonltchPay_a_post;
    w_0_nonltchPay_s_post = paramsStruct.nonLtchPayAggMat_s_post * w_0_nonltchPay_s_post;

    
    if paramsStruct.ind_omitPrePPSNonLtch
        temp_diag_post  =   [temp_diag_post; ...
                            w_0_nonltchPay_a_post; w_0_nonltchPay_s_post];
        temp_diag       =   [temp_diag; ...
                            w_0_nonltchPay_a_post; w_0_nonltchPay_s_post];
    else
        temp_diag_pre   =   [temp_diag_pre; ...
                        w_0_nonltchPay_a_pre; w_0_nonltchPay_s_pre];
        temp_diag_post  =   [temp_diag_post; ...
                            w_0_nonltchPay_a_post; w_0_nonltchPay_s_post];
        temp_diag       =   [temp_diag; ...
                            w_0_nonltchPay_a_pre; w_0_nonltchPay_s_pre; ...
                            w_0_nonltchPay_a_post; w_0_nonltchPay_s_post];
    end
end

W_0         = diag(temp_diag);
W_0_pre     = diag(temp_diag_pre);
W_0_post    = diag(temp_diag_post);

paramsStruct.W_0        = W_0;
paramsStruct.W_0_pre    = W_0_pre;
paramsStruct.W_0_post   = W_0_post;


%%
% Import the variance-covariance matrix needed for GMM standard errors.
if paramsStruct.ind_omitPrePPSNonLtch
    temp_matfile = matfile(fullfile(dataroot, ...
        ['varcovMatrix' paramsStruct.hosp '_omitPrePPSNonLtch']));
else
    temp_matfile = matfile(fullfile(dataroot, ...
        ['varcovMatrix' paramsStruct.hosp]));
end
paramsStruct.bootstrap_var_m_hat = temp_matfile.bootstrap_var_m_hat;

clear temp_*;


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SET UP GLOBAL ARRAYS of PARAMETERS ESTIMATED
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Create a table with param names and whether each param is included.
% Cell array containing parameter names in order
params_name_healthPre = {'muPre'; 'rhoPre'; 'rho1Pre'; 'sigmaPre'; ...
    'mu0Pre'; 'sigma0Pre'};
params_name_healthPost = {'muPost'; 'rhoPost'; 'rho1Post'; 'sigmaPost'; ...
    'mu0Post'; 'sigma0Post'};
params_name_pref = {'gamma'; 'v1A'; 'v0S'; 'v1S'; 'scale'};
params_name_nonLtchPre  = {'zeta0APre'; 'zeta1APre'; 'zeta0SPre'; 'zeta1SPre'};
params_name_nonLtchPost = {'zeta0APost'; 'zeta1APost'; 'zeta0SPost'; 'zeta1SPost'};

params_list = [params_name_healthPre; params_name_healthPost; params_name_pref; ...
        params_name_nonLtchPre; params_name_nonLtchPost];
params_list = cell2table(params_list, ...
    'variablenames', {'param'});
params_list.Properties.RowNames = params_list.param;
params_list.include = ones(numel(params_list.param),1);


% Models a5 expand the health process rhos to make the process time-varying.
switch(estType)
    case({'a3', 'a8', 'a9'})
        params_list.include('rho1Pre')  = 0;
        params_list.include('rho1Post') = 0;
end

% If the type of objective function does not match both discharge shares
% and post-LTCH payments, we don't estimate the paymemts models'
% coefficients.
if ~strcmpi(paramsStruct.objType,'discShares_nonltchPay')
    params_list.include('zeta0APre')  = 0;
    params_list.include('zeta1APre')  = 0;
    params_list.include('zeta0SPre')  = 0;
    params_list.include('zeta1SPre')  = 0;
    params_list.include('zeta0APost') = 0;
    params_list.include('zeta1APost') = 0;
    params_list.include('zeta0SPost') = 0;
    params_list.include('zeta1SPost') = 0;
end

% To omit pre-PPS post-LTCH payments as moments.
if paramsStruct.ind_omitPrePPSNonLtch
    params_list.include('zeta0APre') = 0;
    params_list.include('zeta1APre') = 0;
    params_list.include('zeta0SPre') = 0;
    params_list.include('zeta1SPre') = 0;
end

% Decide what parameters to exclude based on momentsUsed (whether pre-PPS, 
% post-PPS, or both).
switch(momentsUsed)
    case('pre')
        params_list.include('muPost')     = 0;
        params_list.include('rhoPost')    = 0;
        params_list.include('rho1Post')   = 0;
        params_list.include('sigmaPost')  = 0;
        params_list.include('mu0Post')    = 0;
        params_list.include('sigma0Post') = 0;
        params_list.include('zeta0APost') = 0;
        params_list.include('zeta1APost') = 0;
        params_list.include('zeta0SPost') = 0;
        params_list.include('zeta1SPost') = 0;
    case('post')
        params_list.include('muPre')     = 0;
        params_list.include('rhoPre')    = 0;
        params_list.include('rho1Pre')   = 0;
        params_list.include('sigmaPre')  = 0;
        params_list.include('mu0Pre')    = 0;
        params_list.include('sigma0Pre') = 0;
        params_list.include('zeta0APre') = 0;
        params_list.include('zeta1APre') = 0;
        params_list.include('zeta0SPre') = 0;
        params_list.include('zeta1SPre') = 0;    
end

paramsEstimated = params_list.param(params_list.include==1);
paramsStruct.paramsEstimated = paramsEstimated;
retStruct.paramsEstimated    = paramsEstimated;

% Define parameters that depend on the chosen grandver and estimation type
% (contained in version name).
% We need params_all_0 to only contain the parameters to be estimated;
% otherwise the simplex will be set up differently.
params_all_0 = superset_params_all_0(params_list.include==1);
paramsStruct.params_all_0 = params_all_0;

% Manually specify the amount of perturbation to each parameter - make sure
% to update if the full set of parameters changes (eg. any parameter is 
% added to or removed from the model).
params_list.delta = 0.0025 * ones(numel(params_list.param),1);

% Perturbation for each paramter to calculate Jacobian needed for the
% standard errors.
paramsDelta = params_list.delta(params_list.include==1);
paramsStruct.paramsDelta = paramsDelta;
retStruct.paramsPerturb  = paramsDelta;


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SET UP ESTIMATION CONFIGRATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strcmp(estType,'a8')
    %myopic 
    paramsStruct.beta = 0;    
else
    paramsStruct.beta = .9999;
end

%put cap and floor for value function before going into logit form
%
switch(estType)
    case({'a3', 'a5'})
        paramsStruct.valFuncCeil       = 1e2;
        paramsStruct.valFuncFloor_pre  = -1e2;
        paramsStruct.valFuncFloor_post = -2e3;
        paramsStruct.valFuncTrim_back0 = 0;  %do we trim value function at the begining of pre-period backward induction
        paramsStruct.healthGridNumSdev = 3;
    case({'a8', 'a9'})
        paramsStruct.valFuncCeil       = 1e2;
        paramsStruct.valFuncFloor_pre  = -2e2;
        paramsStruct.valFuncFloor_post = -2e3;
        paramsStruct.valFuncTrim_back0 = 1;
        paramsStruct.healthGridNumSdev = 4;
end

%number of core to parallelize
paramsStruct.maxlab=6;

%%
endMsg = sprintf(...
    ['%s -- Version=%s, run=%s:\nFinished setting up params.\n', ...
    'estType: %s.\n', ...
    'estTypeSuffix: %s.\n', ...
    'momentsUsed: %s.\n', ...
    'hosp: %s.\n', ...
    'objType: %s.\n', ...
    'nonLtchPayType: %s.\n', ...
    'No. params to est: %d.\n', ...
    'The params and initial vals for the Simplex Algo\n', ...
    '(these initial vals are not applicable to grid search):\n'],...
    char(datetime('now')),version,run, ...
    paramsStruct.estType, paramsStruct.estTypeSuffix, ...
    paramsStruct.momentsUsed, paramsStruct.hosp, ...
    paramsStruct.objType, paramsStruct.nonLtchPayType, ...
    numel(paramsEstimated));
fprintf('%s', endMsg);

temp_numParams = numel(paramsEstimated);
for temp_index = 1:temp_numParams
    fprintf('%d) %s, initial value = %.4f\n', temp_index, ...
        cell2mat(paramsEstimated(temp_index)), ...
        params_all_0(temp_index));
end
clear endMsg temp_*;